from __future__ import annotations

from typing import List
from tcsoa.gen.DocumentManagement._2011_06.AttributeExchange import ResolveAttrMappingsAndSetPropertiesResponse, ResolveAttrMappingsAndSetPropertiesInfo, ResolveAttrMappingsAndGetPropertiesResponse, ResolveAttrMappingsAndGetPropertiesInfo
from tcsoa.utils import TcService


class AttributeExchangeService(TcService):

    @classmethod
    def resolveAttrMappingsAndGetProperties(cls, info: List[ResolveAttrMappingsAndGetPropertiesInfo]) -> ResolveAttrMappingsAndGetPropertiesResponse:
        """
        This operation processes the metadata exchange mapping information between the client application and
        Teamcenter from the provided input list of ResolveAttrMappingsAndGetPropertiesInfo structure (containing the
        metadata exchange mapping information from the client application).  The operation then gets and returns the
        property  values of the corresponding Teamcenter object from provided input information.
        
        Use cases:
        Metadata exchange between Teamcenter and Microsoft Office Word application
        From the Teamcenter client for Microsoft Office Word 2010, a user opens and checks out a WordX Dataset.  User
        then clicks on the Teamcenter ribbon and clicks on Attribute Exchange> Configurations> Create.  From here user
        can set up the metadata exchange between the properties of the Microsoft Office Word file properties and the
        properties of the Teamcenter object.  
        
        In the configuration, user can set the direction of the metadata exchange either as File to Teamcenter,
        Teamcenter to File, or Two Way.  In this case, user selects Teamcenter to File for the metadata exchange from
        the client to Teamcenter.  User sets up the property mapping by selecting a file property (Comments for
        example) and selecting a Teamcenter object property (object_desc for example), and saves the attribute exchange
        configuration.  User then clicks on Attribute Exchange>Reload button.  The Microsoft Office Word initiates this
        operation and gets Teamcenter object property (object_desc for example) value back.  To verify the client file
        property gets updated, from Microsoft Office Word menu File, select Info (in the left panel), then select
        Properties (in the right panel), then select Show Document Panel.
        """
        return cls.execute_soa_method(
            method_name='resolveAttrMappingsAndGetProperties',
            library='DocumentManagement',
            service_date='2011_06',
            service_name='AttributeExchange',
            params={'info': info},
            response_cls=ResolveAttrMappingsAndGetPropertiesResponse,
        )

    @classmethod
    def resolveAttrMappingsAndSetProperties(cls, info: List[ResolveAttrMappingsAndSetPropertiesInfo]) -> ResolveAttrMappingsAndSetPropertiesResponse:
        """
        This operation processes the metadata exchange mapping information between the client and Teamcenter from the
        provided input list of ResolveAttrMappingsAndSetPropertiesInfo structure (containing the metadata exchange
        mapping information from the client application).  The operation sets the Teamcenter property values based on
        the provided input information.
        
        
        Use cases:
        Metadata exchange between Teamcenter and Microsoft Office Word application
        
        From the Teamcenter client for Microsoft Office Word 2010, a user opens and checks out a WordX Dataset.  User
        then clicks on the Teamcenter ribbon and clicks on Attribute Exchange >Configurations>Create.  From here user
        can set up the metadata exchange between the properties of the Microsoft Office Word file and the properties of
        the Teamcenter object.  
        
        In the configuration, user can set the direction of the metadata exchange either as File to Teamcenter,
        Teamcenter to File, or Two Way.  In this case, user selects File to Teamcenter for the metadata exchange, pick
        a file property (Comment for example), pick a Teamcenter object property (object_desc for example), and save
        the attribute exchange configuration.  From Microsoft Office Word menu File, select Info (left panel), then
        select Properties (right panel)>Show Document Panel.  Update the Comments text box in the Document Properties
        Panel with some text.  Now select Teamcenter ribbon and click on Save.  Save and check in the Dataset.  During
        this process, the Microsoft Office Word initiates this operation and updates Teamcenter object property
        (object_desc for example) value.  User can verify the Teamcenter object property (object_desc for example) by
        login to a Teamcenter client such as Rich Application Client (RAC), do a View properties on the Teamcenter
        object.
        """
        return cls.execute_soa_method(
            method_name='resolveAttrMappingsAndSetProperties',
            library='DocumentManagement',
            service_date='2011_06',
            service_name='AttributeExchange',
            params={'info': info},
            response_cls=ResolveAttrMappingsAndSetPropertiesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class TaskSearchFilter(TcBaseObj):
    """
    A structure representing a string, date or numeric type of Task search filter.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter".
    An example for this input:
    
    StringFilter - User wants to search all the tasks based on any string property on EPMtask like object_type etc.
    
    DateFilter - User wants to search all the tasks based on any date property on EPMTask like due_date etc.
    
    NumericFilter - User wants to search all the tasks based on any numeric property on EPMTask like task_duration etc.
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var startEndRange: The 'gap' used to generate the start and end values.
    """
    searchFilterType: str = ''
    stringValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class TaskSearchFilterField(TcBaseObj):
    """
    A structure containing details about a search filter field.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0


@dataclass
class TaskSearchInput(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var startIndex: This input is used for pagination. User can specify the the start index of the next chunk of tasks
    to return. An example of this input is : User has 100 tasks in inbox and want to get tasks from 51 onwards. So here
    the startIndex would be 51.
    :var maxToReturn: This input is used for pagination. This input specifies the maximum number of found tasks to
    return.
    :var maxToLoad: This input is used for pagination. This input specifies the he maximum number of tasks to load.
    :var searchFilterMap: The map containing the list of search filters for each search filter field.This is a map of
    (string, list(TaskSearchFilter)).
    :var searchSortCriteria: The criteria to use to sort the results. Use this input to specify the EPMTask property to
    be used for sorting the search result. An example for this input is : User wants to sort the output result based on
    "last_mod_date" property etc.
    :var searchSortDirection: The criteria to decide the sort direction of the results.Valid values are "ASC", "DESC".
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response.
    Valid values are "Alphabetical", "Priority".
    :var searchInboxContentType: The criteria to decide the Inbox content type for searching the tasks. Valid values
    are "AllTasks", "MyTasks" and "TasksToTrack".
    """
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: TaskSearchFilterMap = None
    searchSortCriteria: str = ''
    searchSortDirection: str = ''
    searchFilterFieldSortType: str = ''
    searchInboxContentType: str = ''


@dataclass
class TaskSearchResponse(TcBaseObj):
    """
    A service response structure containing search results.
    
    :var searchResults: The list of business objects obtained after performing a search.
    :var totalFound: The total number of business objects found.
    :var totalLoaded: The total number of business objects loaded.
    :var searchFilterMap: The map containing the list of search filters for each search filter field based on the
    search results. This is a map of (string, list(TaskSearchFilter)).
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var serviceData: Service data containing found objects, errors, etc.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap: TaskSearchFilterMap = None
    searchFilterCategories: List[TaskSearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    serviceData: ServiceData = None


"""
TaskSearchFilterMap is a map of search filter name (key) and TaskSearchFilter structure(values).
"""
TaskSearchFilterMap = Dict[str, List[TaskSearchFilter]]



from __future__ import annotations

from tcsoa.gen.Internal.Core._2007_01.DataManagement import GetOrganizationInformationInputData, GetAttributeValuesInputData, GetAttributeValuesResponse, GetOrganizationInformationResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getAttributeValues(cls, classAttributeValues: List[GetAttributeValuesInputData]) -> GetAttributeValuesResponse:
        """
        This operation retrieves the values of an attribute from the database for all the instances of a class.
        """
        return cls.execute_soa_method(
            method_name='getAttributeValues',
            library='Internal-Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'classAttributeValues': classAttributeValues},
            response_cls=GetAttributeValuesResponse,
        )

    @classmethod
    def getOrganizationInformation(cls, options: List[GetOrganizationInformationInputData]) -> GetOrganizationInformationResponse:
        """
        This operation  retrieves the whole organization group hierarchy with groups at all the levels.  The hierarchy
        could include Role objects added to each groups as well as GroupMember objects as well by specifying options in
        input GetOrganizationInformationInputData object to include role objects and group member objects. The group
        name in GetOrganizationInformationInputData object should always be empty and the option to return only first
        level subgroups in GetOrganizationInformationInputData object is not implemented and should be ignored.
        """
        return cls.execute_soa_method(
            method_name='getOrganizationInformation',
            library='Internal-Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'options': options},
            response_cls=GetOrganizationInformationResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetStylesheetPerPagePref(TcBaseObj):
    """
    A set of preferences related to stylesheet information specified for each input object.
    
    :var returnThumbnailTickets: If true the thumbnail FMS file tickets are returned.
    :var stylesheetFormat: The stylesheet format to be returned. Legal values are "XRTOnly", "HTMLOnly", or
    "HTMLPreferred".
    XRTOnly - Only XRT format is returned, no attempt to return HTML is made.
    HTMLOnly - Only the HTML format is returned, even in error situations.
    HTMLPreferred - HTML format is returned, except in error situations or when unsupported element or rendering hints
    are found in XRT.
    :var processEntireXRT: If true, this operation will process the data for the entire XRT stylesheet.  If false, this
    operation will only process the target page which is defined in the input structure.
    """
    returnThumbnailTickets: bool = False
    stylesheetFormat: str = ''
    processEntireXRT: bool = False


@dataclass
class GetStylesheetPerPageResponse(TcBaseObj):
    """
    Output structure for the getStylesheetPerPage operation.
    
    :var outputs: A list of StylesheetPerPageOutput objects, one for each input object.
    :var serviceData: This service data contains any partial errors which may have been encountered during processing.
    The partial error client Ids match the client Id for the input which failed.
    """
    outputs: List[StylesheetPerPageOutput] = ()
    serviceData: ServiceData = None


@dataclass
class StylesheetPerPageInputData(TcBaseObj):
    """
    A set of input data for which to retrieve stylesheet information. One input object corresponds to one Teamcenter
    object for which to retrieve data.
    
    :var clientId: This is a unique identifier in order to look up Partial Errors and find corresponding output
    structures.
    :var boName: The business object type name. It is used to determine the stylesheet to load. If the boReference
    field is set this field is ignored.
    :var boReference: The workspace object for which to load a stylesheet. If this field is set the boName field is
    ignored.
    :var stylesheetType: The stylesheet type to search for. Supported values are: "Summary", "Form", "Create",
    "SaveAs", or "Properties".
    :var targetPage: The page in the XRT stylesheet to get data for. This field is ignored if the parameter
    processEntireXRT in the preference is set to true.
    Name of the page in the XRT stylesheet to get the data for. The value can either be the page title key value, page
    title or page name, for example "web_xrt_Overview".  If the page refered to is not found or not visible, then first
    visible page in the XRT will be returned.
    If empty string is specified, the first visible page in the XRT
    will be returned.
    """
    clientId: str = ''
    boName: str = ''
    boReference: BusinessObject = None
    stylesheetType: str = ''
    targetPage: str = ''


@dataclass
class StylesheetPerPageOutput(TcBaseObj):
    """
    The StylesheetPerPageOutput structure contains all the data required for a client to present the stylesheet to the
    user for the input object.
    
    :var clientId: This is the client Id which was sent in on input.
    :var xrtStylesheet: The XRT style sheet, if XRT is to be returned.
    :var htmlStylesheet: The HTML stylesheet, if HTML is to be returned.
    :var childrenMap: The map (string, list of business object) where the key is  property name and the value is a list
    of child objects.
    These child objects also needs to be render in the UI along with boReference business object.
    :var thumbnailFileTicketMap: A map (business object, string) of child business objects to its thumbnail file ticket.
    :var processedPage: The name of the page that was processed. If the operation was invoked with processEntireXRT set
    to true, this field is empty.  If the operation was invoked with processEntireXRT set to false, then this field is
    populated with the page that was processed.  Typically the value of the processedPage would be the same as the
    targetPage, except when the targetPage value is empty or not found in the stylesheet.  In that case, then the first
    page in the stylesheet is processed.
    :var visiblePages: A list of visible pages in the XRT. The values can be either the page title key or page title or
    page name. The names from this list can be used in a subsequent call to this operation to query for the data for
    that page.
    """
    clientId: str = ''
    xrtStylesheet: str = ''
    htmlStylesheet: str = ''
    childrenMap: StylesheetPerPageChildrenMap = None
    thumbnailFileTicketMap: FileTicketMap2 = None
    processedPage: str = ''
    visiblePages: List[str] = ()


"""
Map from business object to thumbnail file ticket.
"""
FileTicketMap2 = Dict[BusinessObject, str]


"""
Map of property names to child objects.
"""
StylesheetPerPageChildrenMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AutoPositionComponentByCSYSResponse(TcBaseObj):
    """
    Response from autoPositionComponentByCSYS
    
    :var serviceData: The service data containing partial errors if any.
    :var possibleSourceCSYSs: The list of CSYSs from parent component, if there are multiple possible CSYSs appropriate
    for positioning.
    :var possibleTargetCSYSs: The list of CSYSs from child component, if there are multiple possible CSYSs appropriate
    for positioning.
    """
    serviceData: ServiceData = None
    possibleSourceCSYSs: List[AutoPositioningOutput] = ()
    possibleTargetCSYSs: List[AutoPositioningOutput] = ()


@dataclass
class AutoPositioningOutput(TcBaseObj):
    """
    AutoPositioningOutput
    
    :var strCSYSBOMLine: CSYS BOMLine.
    :var strCSYSBOMLineName: Name of the CSYS.
    """
    strCSYSBOMLine: str = ''
    strCSYSBOMLineName: str = ''



from __future__ import annotations

from tcsoa.gen.StructureManagement._2010_04.StructureSearch import SearchExpressionSet
from tcsoa.gen.StructureManagement._2008_05.StructureSearch import SearchScope, StructureSearchResultResponse
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def startSearch(cls, scope: SearchScope, searchExpression: SearchExpressionSet) -> StructureSearchResultResponse:
        """
        Start searching a structure for a given search expression within the scope specified
        """
        return cls.execute_soa_method(
            method_name='startSearch',
            library='StructureManagement',
            service_date='2010_04',
            service_name='StructureSearch',
            params={'scope': scope, 'searchExpression': searchExpression},
            response_cls=StructureSearchResultResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.GlobalMultiSite._2020_01.Briefcase import RemoveMarkOTResponse, AddMarkOTResponse, QueryMarkOTResponse, GetObjectsLockInfoResponse
from typing import List
from tcsoa.gen.Internal.GlobalMultiSite._2020_01.OwnershipRecovery import OtTransactionResponse, RecoverOwnershipResponse, OtSearchInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class BriefcaseService(TcService):

    @classmethod
    def queryMarkOT(cls, objects: List[str], userId: str, targetSiteId: int, needHelperObjects: bool, appId: str) -> QueryMarkOTResponse:
        """
        This operation queries for marks of briefcase ownership transfer. All the input parameters are the search
        criteria.
        
        Use cases:
        &bull;    A user logs into Teamcenter RAC and opens the view "Briefcase Ownership Transfer".
        &bull;    The view "Briefcase Ownership Transfer" calls this operation to query the OT marks owned by the
        current user. 
        &bull;    The operation returns all found OT marks owned by the logged in user.
        &bull;    The returned OT marks are displayed in "Briefcase Ownership Transfer" view.
        """
        return cls.execute_soa_method(
            method_name='queryMarkOT',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='Briefcase',
            params={'objects': objects, 'userId': userId, 'targetSiteId': targetSiteId, 'needHelperObjects': needHelperObjects, 'appId': appId},
            response_cls=QueryMarkOTResponse,
        )

    @classmethod
    def removeMarkOTForCurrentUser(cls, objects: List[str], appId: str) -> RemoveMarkOTResponse:
        """
        This operation removes the marks for briefcase ownership transfer. The following search criteria are used to
        search for marks and remove them.
        &bull;    A list of business object UID to search marks on them. 
        &bull;    Owned by the current logged user.
        &bull;    Application ID. If the value is empty, the operation searches marks owned by all applications.
        
        Use cases:
        &bull;    A user logs into Teamcenter RAC and open the view "Briefcase Ownership Transfer". 
        &bull;    Business objects with marks for Briefcase ownership transfer are display in this view.
        &bull;    A user chooses one or multiple objects and clicks the "Remove" button.
        &bull;    RAC calls this operation and send the selected business objects as input.
        &bull;    Application ID with the value "BC" is the input parameter for this operation too.
        &bull;    The operation removes all the marks found on the given business objects and the application id.
        &bull;    After this operation is completed, RAC calls another service operation "queryMarkOT" to refresh marks
        in the view "Briefcase Ownership Transfer".
        """
        return cls.execute_soa_method(
            method_name='removeMarkOTForCurrentUser',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='Briefcase',
            params={'objects': objects, 'appId': appId},
            response_cls=RemoveMarkOTResponse,
        )

    @classmethod
    def addMarkOTForCurrentUser(cls, objects: List[str], targetSiteId: int, appId: str) -> AddMarkOTResponse:
        """
        This operation adds marks to the specific business objects to lock them for briefcase ownership transfer. The
        marks are owned by the current user only. After the marks are applied, other users cannot modify the objects or
        mark them for ownership. The marks exist after the owning user logs out Teamcenter. The marks can be removed
        after briefcase export is completed or be removed explicitly from Briefcase ownership transfer view.
        
        Use cases:
        &bull;    A user logs in to Teamcenter RAC.
        &bull;    In "My Teamcenter" application, one or multiple items are selected. In right-click context menu
        window, the menu "Mark for Ownership Transfer" is clicked.
        &bull;    RAC first calls the service operation "getObjectsLockInfo" to get locks from the selected business
        objects.
        &bull;    If no locks are found, this operation is called to add marks on these selected business objects.
        &bull;    The operation first adds T locks on these objects. Then it adds ownership transfer marks on these
        objects.
        &bull;    The operation returns the success and failure information.
        &bull;    RAC displays the failure information.
        &bull;    The successful locked business objects are displayed in the view "Briefcase Ownership Transfer".
        """
        return cls.execute_soa_method(
            method_name='addMarkOTForCurrentUser',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='Briefcase',
            params={'objects': objects, 'targetSiteId': targetSiteId, 'appId': appId},
            response_cls=AddMarkOTResponse,
        )

    @classmethod
    def getObjectsLockInfo(cls, objects: List[str]) -> GetObjectsLockInfoResponse:
        """
        This operation queries for modification locks on the specific business objects.
        
        Use cases:
        &bull;    A user logs in to Teamcenter RAC.
        &bull;    In My Teamcenter application, he selects one or several items and right click. A context menu window
        pops up and he clicks the menu "Mark for Ownership Transfer".
        &bull;    RAC calls the operation and uses the selected items as input parameter.
        &bull;    The operation returns the found M locks and T locks on the specified business objects.
        """
        return cls.execute_soa_method(
            method_name='getObjectsLockInfo',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='Briefcase',
            params={'objects': objects},
            response_cls=GetObjectsLockInfoResponse,
        )


class OwnershipRecoveryService(TcService):

    @classmethod
    def recoverOwnership(cls, transactionId: str, dryRun: bool) -> RecoverOwnershipResponse:
        """
        Recovers the ownership of all the objects associated with the briefcase transaction that includes ownership
        transfer (OT). This operation makes the objects as local at exporting site and replica at importing site. The
        operation also supports dry run mode to review the objects for which ownership would be changed before actually
        implementing the ownership changes.
        This will recover the ownership only if that import/export briefcase transaction with OT was performed by the
        current user. DBA users can recover the ownership transferred in transactions done any user.
        
        Use cases:
        This operation is used to recover ownership of objects associated with a briefcase transaction that includes OT
        when that ownership has been inadvertently transferred. Id of the briefcase transaction is used to recover the
        ownership of its associated objects.
        """
        return cls.execute_soa_method(
            method_name='recoverOwnership',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='OwnershipRecovery',
            params={'transactionId': transactionId, 'dryRun': dryRun},
            response_cls=RecoverOwnershipResponse,
        )

    @classmethod
    def recoverOwnershipUsingBriefcase(cls, briefcaseUid: str, dryRun: bool) -> RecoverOwnershipResponse:
        """
        Recovers the ownership of all the objects associated with a briefcase transaction that includes ownership
        transfer (OT). This operation makes the objects as local at exporting site and replica at importing site. The
        operation also supports dry run mode to review the objects for which ownership would be changed before actually
        implementing the ownership changes.
        This will recover the ownership only if the briefcase was exported/imported by the current user. DBA users can
        recover the ownership of objects associated with briefcases exported/imported by any user.
        
        Use cases:
        This operation is used to recover ownership of objects associated with a briefcase transaction that includes OT
        when that ownership has been inadvertently transferred. UID of the briefcase is used to recover the ownership
        of its associated objects.
        """
        return cls.execute_soa_method(
            method_name='recoverOwnershipUsingBriefcase',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='OwnershipRecovery',
            params={'briefcaseUid': briefcaseUid, 'dryRun': dryRun},
            response_cls=RecoverOwnershipResponse,
        )

    @classmethod
    def deleteOtTransaction(cls, transactionId: str) -> ServiceData:
        """
        Deletes the briefcase transaction that includes OT from the database. Once deleted, you cannot recover
        ownership of the objects associated with the specified transaction.
        Non-DBA users can only delete the transactions performed by themselves. DBA users can delete any transaction.
        
        Use cases:
        This operation is used to delete a briefcase transaction record that was carried out with OT from database.
        Once deleted, ownership recovery cannot be done for objects associated with that briefcase transaction.
        """
        return cls.execute_soa_method(
            method_name='deleteOtTransaction',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='OwnershipRecovery',
            params={'transactionId': transactionId},
            response_cls=ServiceData,
        )

    @classmethod
    def findOtTransactions(cls, otSearchInfo: OtSearchInfo) -> OtTransactionResponse:
        """
        Search briefcase transactions which include ownership transfer of objects by giving the information or criteria
        for transaction search. For non-DBA users, the search result will only contain the transactions done by the
        user i.e. briefcase exported/imported by the current user. DBA users will get the briefcase transactions done
        by all users.
        Search can be done on following criteria &ndash;
        - siteId - All briefcases with OT exported to or imported from the site with the given id will be listed. If
        the siteId is not given, then the briefcases exported from/ imported to current site are listed.
        - transactionType - Allowed values for transaction type are&ndash; import, export or empty string. This will
        limit the search results to only imported or exported briefcases to given site.
        - startDate - Specifies the earliest date and time a briefcase transaction must have occurred to be included in
        the search result.
        - endDate - Specifies the latest date and time a briefcase transaction must have occurred to be included in the
        search result.
        - briefcaseName - Name of the briefcase file.
        
        
        
        Use cases:
        This operation is used to find briefcases transactions(export/import) that include ownership transfer of
        Teamcenter objects. You can select the required briefcase for ownership recovery from the list returned based
        on the input search criteria.
        """
        return cls.execute_soa_method(
            method_name='findOtTransactions',
            library='Internal-GlobalMultiSite',
            service_date='2020_01',
            service_name='OwnershipRecovery',
            params={'otSearchInfo': otSearchInfo},
            response_cls=OtTransactionResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ProjectManagement._2008_06.ScheduleManagement import SubMasterMetaData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Internal.ProjectManagement._2009_10.ScheduleManagement import TranslationDataRequest, TranslationPreferences, TranslationSchedules, TranslationModelClass
from dataclasses import dataclass


@dataclass
class TranslationDataContainer(TcBaseObj):
    """
    Internal data structure for moving POM objects between C/C++ and Java translator
    
    :var schedules: list of TranslationDataSchedule structure
    :var modelMetaData: Structure containing model classes
    :var preferences: list of string value containers of preferences
    :var requestData: request data
    """
    schedules: TranslationSchedules = None
    modelMetaData: TranslationModelMetaData = None
    preferences: TranslationPreferences = None
    requestData: TranslationDataRequest = None


@dataclass
class TranslationModelMetaData(TcBaseObj):
    """
    Schedule Schema
    
    :var modelClass: model class
    :var subMasterRelationship: Metadata containing relationship of sub-schedule to Master-Schedules
    """
    modelClass: List[TranslationModelClass] = ()
    subMasterRelationship: List[SubMasterMetaData] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class SearchFilter(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter";.
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var startEndRange: The 'gap' used to generate the start and end values
    """
    searchFilterType: SearchFilterType = None
    stringValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchFilterField(TcBaseObj):
    """
    A structure containing details about a search filter field.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0


@dataclass
class SearchInput(TcBaseObj):
    """
    A structure containing input search criteria
    
    :var providerName: The name of the search provider.  This is the RuntimeBusinessObject type name.
    :var searchCriteria: The criteria used to perform search (string/string). For example, for object set search, the
    search criteria are parentUid and object set source string.
    :var startIndex: The start index to return the search results
    :var maxToReturn: The maximum number of search results to return
    :var maxToLoad: The maximum number of search results to load
    :var searchFilterMap: The map (string,list of SearchFilter) containing the list of search filters for each search
    filter field.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response. The
    acceptable values are: "Alphabetical","Priority".
    :var attributesToInflate: The list of attributes to inflate.
    """
    providerName: str = ''
    searchCriteria: StringMap = None
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: SearchFilterMap = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: SearchFilterFieldSortType = None
    attributesToInflate: List[str] = ()


@dataclass
class SearchResponse(TcBaseObj):
    """
    A service response structure containing search results.
    
    :var searchResults: The list of business objects obtained after performing a search.
    :var totalFound: The total number of business objects found.
    :var totalLoaded: The total number of business objects loaded.
    :var searchFilterMap: The map (string,list of SearchFilter) containing the list of search filters for each search
    filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var serviceData: The service data object.
    """
    searchResults: List[BusinessObject] = ()
    totalFound: int = 0
    totalLoaded: int = 0
    searchFilterMap: SearchFilterMap = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    serviceData: ServiceData = None


@dataclass
class SearchSortCriteria(TcBaseObj):
    """
    The criteria to use to sort the results.
    
    :var fieldName: The name of the field on which to perform the sorting.
    :var sortDirection: The direction in which the sorting needs to be perfomed - 'ASC' or 'DESC'.
    """
    fieldName: str = ''
    sortDirection: SortDirection = None


class SearchFilterFieldSortType(Enum):
    """
    An enumeration to determine how fields used in filters need to be sorted. The fields can be sorted alphabetically
    in ascending order or in priority order from highest to lowest.
    """
    Alphabetical = 'Alphabetical'
    Priority = 'Priority'


class SearchFilterType(Enum):
    """
    An enumeration for different types of search filters like string, date, numeric.
    """
    StringFilter = 'StringFilter'
    DateFilter = 'DateFilter'
    NumericFilter = 'NumericFilter'


class SortDirection(Enum):
    """
    An enumeration indicating whether the sorting needs to be performed in ascending or descending order.
    """
    ASC = 'ASC'
    DESC = 'DESC'


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap = Dict[str, List[SearchFilter]]


"""
Map from string to string
"""
StringMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2017_06.RequirementsManagement import BaselineInputDataAsync, ExportToApplicationInputData4, BaselineInputData
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput, SearchInput, SaveColumnConfigData
from tcsoa.gen.Internal.AWS2._2016_03.RequirementsManagement import ExportToApplicationResponse2
from tcsoa.gen.Internal.AWS2._2016_03.UiConfig import GetOrResetUIColumnConfigInput
from tcsoa.gen.Internal.AWS2._2017_06.UiConfig import GetOrResetUIColumnConfigResponse
from tcsoa.gen.Internal.AWS2._2016_04.DataManagement import GetStyleSheetIn
from tcsoa.gen.Internal.AWS2._2017_06.DataManagement import LoadViewModelForEditingInfo, GetViewerDataResponse, LoadViewModelForEditingResponse, SaveViewModelEditAndSubmitInfo, GetViewerDataIn, GetViewModelPropsResponse, GetStyleSheetResponse, SaveViewModelEditAndSubmitResponse
from tcsoa.gen.Internal.AWS2._2017_06.FullTextSearch import IdentifyImpactedObjectsResponse, IdentifyImpactedObjInput
from typing import List
from tcsoa.gen.Internal.AWS2._2017_06.EffectivityManagment import ReleaseStatusEffectivityInput
from tcsoa.gen.Internal.AWS2._2017_06.Finder import SearchResponse3
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class EffectivityManagmentService(TcService):

    @classmethod
    def addOrRemoveRelStatusEffectivities(cls, input: ReleaseStatusEffectivityInput) -> ServiceData:
        """
        This operation adds or removes shared Effectivity objects to or from the input ReleaseStatus.
        
        Exceptions:
        >126231 The effectivities cannot be updated on ReleaseStatus.
        """
        return cls.execute_soa_method(
            method_name='addOrRemoveRelStatusEffectivities',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='EffectivityManagment',
            params={'input': input},
            response_cls=ServiceData,
        )


class UiConfigService(TcService):

    @classmethod
    def getOrResetUIColumnConfigs2(cls, getOrResetUiConfigsIn: List[GetOrResetUIColumnConfigInput]) -> GetOrResetUIColumnConfigResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes column configuration information. 
        If the resetColumnConfig flag is "True", this operation deletes the column configuration of the input scope and
        then queries the new effective UI column configuration for the login user. This operation will clear the login
        user column configurations.
        
        Use cases:
        Request UI Configuration(s) based on the current login user
        Client requests the column information for one or more client scopes using this operation and scope as login
        user.
        
        Request UI Configuration(s) based on a specific Teamcenter scope
        Client requests the column information for one or more client scopes using this operation and scope as Role,
        Site or Group.
        
        Request to reset UI Column Configuration(s) based on the current login user
        If a client needs to reset the column information for login user scope, they can use this operation. The new
        effective UI column configuration will be retrieved for the login user.
        """
        return cls.execute_soa_method(
            method_name='getOrResetUIColumnConfigs2',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='UiConfig',
            params={'getOrResetUiConfigsIn': getOrResetUiConfigsIn},
            response_cls=GetOrResetUIColumnConfigResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def getStyleSheet3(cls, processEntireXRT: bool, input: List[GetStyleSheetIn]) -> GetStyleSheetResponse:
        """
        This operation returns the stylesheets and the required data to present that stylesheet for each input object.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getStylesheet call is invoked and the stylesheet is processed in order to display the
        data to the user.
        
        Use Case 2: List view with Summary
        When an object is selected in the ActiveWorkspace navigator, when in list view with summary mode, the
        getStyleSheet operation is invoked, and the returned data is parsed and processed in order to present the data
        to the user in the Summary panel.
        
        Use Case 3: Show Object Info
        When an object is selected in ActiveWorkspace, a user may choose the "Show Object Info" command.  When
        executed, a panel slides out from the right hand side of the application, and the selected objects data is
        presented.  In order to populate the panel, the getStylesheet call is invoked and the returned data is parsed
        and processed in order to build the UI.
        """
        return cls.execute_soa_method(
            method_name='getStyleSheet3',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='DataManagement',
            params={'processEntireXRT': processEntireXRT, 'input': input},
            response_cls=GetStyleSheetResponse,
        )

    @classmethod
    def getViewModelProperties(cls, objects: List[BusinessObject], attributes: List[str]) -> GetViewModelPropsResponse:
        """
        This service operation is provided to get property values of instances outside of the current object property
        policy for a particular business object.This operation is an enhanced version of the getProperties service
        operation and allows for retrieving values of dynamic compound properties as well.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getViewModelProperties call is executed to fetch required properties.
        """
        return cls.execute_soa_method(
            method_name='getViewModelProperties',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='DataManagement',
            params={'objects': objects, 'attributes': attributes},
            response_cls=GetViewModelPropsResponse,
        )

    @classmethod
    def getViewerData(cls, inputs: GetViewerDataIn) -> GetViewerDataResponse:
        """
        This operation returns the dataset, file and relevant viewer data for given combination of input object,
        dataset  and direction by processing the viewer preference set by Teamcenter Administrator. Datasets associated
        with input object are retrieved and sorted based on the values of viewer preference. Thereafter, appropriate
        dataset is identified using the input dataset and direction. The corresponding file and viewer information
        along with dataset is returned in the response.
        """
        return cls.execute_soa_method(
            method_name='getViewerData',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=GetViewerDataResponse,
        )

    @classmethod
    def loadViewModelForEditing(cls, inputs: List[LoadViewModelForEditingInfo]) -> LoadViewModelForEditingResponse:
        """
        This method ensures that the properties can be edited, and returns the last save date of the related objects
        for optimistic edit. This service operation is similar to loadDataForEditing service operation, the difference
        is that this operation saves the ViewModelObject instead of model object, and this operation supports dynamic
        compound properties. If the property is dynamic compound property, this operation checks if the traversal path
        on server is same as the input traversal path , if it is not the same, then editing is not allowed.
        """
        return cls.execute_soa_method(
            method_name='loadViewModelForEditing',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=LoadViewModelForEditingResponse,
        )

    @classmethod
    def saveViewModelEditAndSubmitWorkflow(cls, inputs: List[SaveViewModelEditAndSubmitInfo]) -> SaveViewModelEditAndSubmitResponse:
        """
        This operation saves the modified view model properties for the given input objects and submits the objects to
        a workflow. The workflow is submitted only if all of the save operations are successful. If the save fails for
        a single object  none of the input objects will be submitted to a workflow. This operation is similar to the
        saveEditAndSubmitToWorkflow operation, the difference is that this operation saves the ViewModelObject instead
        of model object, and this operation supports dynamic compound properties.
        
        Use cases:
        User can modify the object(s) properties and submit the object(s) to workflow in one operation. This operation
        first saves the modified properties and then initiates the workflow process for all input objects.
        """
        return cls.execute_soa_method(
            method_name='saveViewModelEditAndSubmitWorkflow',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveViewModelEditAndSubmitResponse,
        )


class FullTextSearchService(TcService):

    @classmethod
    def identifyImpactedObjects(cls, impactedObjectsInput: IdentifyImpactedObjInput) -> IdentifyImpactedObjectsResponse:
        """
        The operation identifies the siblings of Revisionable Objects that have been added/modified/deleted.
        
        The identified siblings are marked in the accountability table with the TIE sync status provided and returned
        if requsted
        
        Based on the value of the addModDelBitMask the following steps will be executed. 
        Steps:
        1.  Queries for siblings of deleted objects from scratch table. Gets their parent Item objects from
        accountability table impacted island if they exist. For the Item objects found, it fetches all the Revisionable
        Objects and adds them to the response.
        2.  Queries for siblings of newly added objects from scratch table. Filters out non Revisionable Objects from
        the list.
        Adds the Revisionable Objects to the reponse.
        3.  Queries for any Modified islands from the accountability table. If modified island belongs to an
        Revisionable Objects, then queries for the Revisionable Objects siblings and adds to the response.
        
        Finally the response will contain the impacted Revisionable Objects due to adds/modify/deletes.
        
        Use cases:
        Use Cases:
        Use Case 1: TcFtsIndexer Sync
        
        Whenever Revisionable Objects are indexed  the Revision rule slector information (Working, Latest Working,
        etc&hellip;) is also indexed on each Revisionable Object. So whenever  a new Revisionable Object  is
        added/deleted/modified that impacts all the Revisionable Object associated with its parent Item and the
        revision rule selector information need to be regenerated. This operation will help in identifying the impacted
        objects and they can be re-indexed to sync the index data .
        """
        return cls.execute_soa_method(
            method_name='identifyImpactedObjects',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='FullTextSearch',
            params={'impactedObjectsInput': impactedObjectsInput},
            response_cls=IdentifyImpactedObjectsResponse,
        )


class FinderService(TcService):

    @classmethod
    def performSearchViewModel(cls, searchInput: SearchInput, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData, inflateProperties: bool) -> SearchResponse3:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures.
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation. 
        
        RuntimeBusinessObject 
        ---- Fnd0BaseProvider (foundation template) 
        -------- Fnd0GetChildrenProvider(foundation template) 
        -------- Awp0FullTextSearchProvider (aws template) 
        -------- Awp0TaskSearchProvider (aws template)
         -------- Aos0TestProvider (aosinternal template) 
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its
        results. 
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object.
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects. 
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type (Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile 
        A search request will be made by passing in the needed criteria. In this case the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown. 
        Use Case 2: User performs search by entering a keyword in global search input box.
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective
        columns for the client scope URI passed in through column config input. The results are finally shown in the
        view type selected in client. If table view is selected the returned effective columns will be shown. 
        Use Case 3: User tries to save new column configurations after performing search. 
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the provider
        name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the client
        scope URI passed in through column config input. The results are finally shown in the view type selected in
        client. If table view is selected the returned effective columns will be shown. 
        ---
        
        ---
        """
        return cls.execute_soa_method(
            method_name='performSearchViewModel',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData, 'inflateProperties': inflateProperties},
            response_cls=SearchResponse3,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def createBaseline(cls, input: List[BaselineInputData]) -> ServiceData:
        """
        This operation creates baseline of the selected business object.The selected object can be of type ItemRevision
        or Awb0Element .Baseline is a mechanism to to preserve the current state of the object in time . This is a
        synchronous operation and it returns back immediately with partial errors if any. This operation calls
        createBaselineAsync (asynchronous service operation) which performs the actual baseline creation. When the
        business object selected for baselining is a structure occurrence of type Awb0Element, new baseline revision is
        created for all the occurrences in that structure. When the selected business object is of type ItemRevision,
        then a single baseline revision is created. User is notified after baseline creation is complete.
        
        This operation is for Teamcenter internal use and is unpublished.
        
        Use cases:
        1. User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure.
        User selects an occurrence and in Tool and Info command bar, clicks on 'Save As or Revise' option. This will
        show up a panel with tab options as  "Revision" and  "New". In Revision tab, 'Save as Baseline' checkbox is
        present. After selecting this checkbox, 'Baseline Template' dropdown box appears. The user selects a baseline
        template from dropdown followed by a click on "Save" button, resulting into baseline creation. Irrespective of
        the selected occurrence, after clicking "Save"; the entire structure ie. top most parent occurrence and all its
        children are baselined. When the baseline creation completes, a notification is sent to the client.
        
        2. User selects an ItemRevision in non ACE view and in Tool and Info command bar clicks on 'Save As or Revise'
        option. This will show up a panel with tab options as  "Revision" and  "New". After selecting this checkbox,
        'Baseline Template' dropdown box appears. In Revision tab, 'Save as Baseline' checkbox is present. The user
        selects a baseline template from dropdown followed by a click on "Save" button, resulting into baseline
        creation for this item revision. When the baseline creation completes, a notification is sent to the client.
        """
        return cls.execute_soa_method(
            method_name='createBaseline',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def createBaselineAsync(cls, input: List[BaselineInputDataAsync]) -> None:
        """
        This operation creates baseline of the selected business object.The selected object can be of type ItemRevision
        or Awb0Element .Baseline is a mechanism to to preserve the current state of the object in time . This is
        asynchronous operation and is invoked by synchronous service operation createBaseline . createBaselineAsync
        operation performs the actual baseline creation. When the business object selected for baselining is a
        structure occurrence of type Awb0Element, new baseline revision is created for all the occurrences in that
        structure. When the selected business object is of type ItemRevision, then a single baseline revision is
        created. User is notified after baseline creation is complete.
        
        Use cases:
        1. User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure.
        User selects an occurrence and in Tool and Info command bar, clicks on 'Save As or Revise' option. This will
        show up a panel with tab options as  "Revision" and  "New". In Revision tab, 'Save as Baseline' checkbox is
        present. After selecting this checkbox, 'Baseline Template' dropdown box appears. The user selects a baseline
        template from dropdown followed by a click on "Save" button, resulting into baseline creation. Irrespective of
        the selected occurrence, after clicking "Save"; the entire structure ie. top most parent occurrence and all its
        children are baselined. When the baseline creation completes, a notification is sent to the client.
        
        2. User selects an ItemRevision in non ACE view and in Tool and Info command bar clicks on 'Save As or Revise'
        option. This will show up a panel with tab options as  "Revision" and  "New". After selecting this checkbox,
        'Baseline Template' dropdown box appears. In Revision tab, 'Save as Baseline' checkbox is present. The user
        selects a baseline template from dropdown followed by a click on "Save" button, resulting into baseline
        creation for this item revision. When the baseline creation completes, a notification is sent to the client.
        """
        return cls.execute_soa_method(
            method_name='createBaselineAsync',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=None,
        )

    @classmethod
    def exportAsync2(cls, input: ExportToApplicationInputData4) -> None:
        """
        This operation exports the selected occurrences in the structure asynchronously to an application such as
        MSExcel and MSWord. The behavior of export operation is controlled by applicationFormat and
        objectTemplateInputs arguments specified in ExportInputDataAsync2 structure. If user selects the "include
        attachments" option, then this operation can export a .zip file that will include all the attachments
        associated with the selected object along with the document. The objects may be checked out during the export.
        
        Following are supported application formats from input.
        
        - MSExcelLive - This mode provides the capability of editing of properties of underlying ItemRevision in
        MSExcel and save the changes to Teamcenter. The generated sheet allows user to login and change the properties
        from MSExcel application. The changes will be saved to Teamcenter when user exits out of the cell in MSExcel
        application.  Teamcenter extensions for Microsoft Office should be installed on the client machine to support
        this application format.
        - MSExcel - This mode provides the capability of exporting objects to static MSExcel application. 
        - MSExcelReimport - This mode provides the capability of exporting objects to MSExcel application for reimport
        purpose. The generated sheet can be used to add/remove structure elements or to update the properties on the
        SpecElementRevision. The generated sheet should be reimported back to Teamcenter. Office Client should be
        installed on the client machine to support this application format.
        - MSWordXMLLive - This mode provides the capability of structural editing of SpecElementRevision in MSWord.
        Users can edit the rich text of Requirements and perform structural changes to the exported Requirements in the
        word document. Office Client should be installed on the client machine to support this application format.
        
        
        
        Use cases:
        - User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure. User selects an occurrence and clicks on the Export to Excel button in the
        toolbar. This will show a panel with options to "View", "Live Edit" and "Edit and Import". This panel allows
        user to select a template and if user clicks on Export button, an Excel sheet will be generated. If user has
        selected a single occurrence and clicks on export, then the occurrence and all its children are exported to
        MSExcel. If user has selected multiple occurrences and clicks on export, then the selected occurrences are
        exported to MSExcel.
        - User selects single or multiple objects in Active Workspace Client. User clicks on the Export to Excel button
        in the toolbar. User can select View, Edit or Reimport option. Depending upon the option selected, a static
        MSExcel report or a Live MSExcel sheet or a sheet is generated that can be reimported back to Teamcenter. 
        - User selects an ItemRevision and clicks on the Export to Excel button on the toolbar. If user selects "Check
        out" option then selected ItemRevision and its children will be checked out and exported to MSExcel.
        - User selects single or multiple objects in ActiveWorkspace Client. User selects the background checkbox in
        the export panel and clicks on the Export to Excel button in the toolbar. This will invoke the Export to Excel
        using the async service options. When the Export to Excel completes, a notification is sent to the client to
        download the Excel file. The generated file is also attached to the MSExcelX dataset.
        - User selects the "apply_bom_crawling" LOV value on the ExcelTemplate and selects an occurrence in ACE and
        clicks on Export to Excel button in the toolbar. This will crawlback from the selected object to its parent's
        recursively till its top line and the exported structure will export the selected line along with its parents.
        - User selects single or multiple objects in ActiveWorkspace Client. User selects the background checkbox in
        the export panel and clicks on the Export to Word button in the toolbar. This will invoke the Export to MSWord
        using the async service options. When the Export to MSWord completes, a notification is sent to the client to
        download the file.
        - User selects single or multiple objects in Active Workspace Client, and clicks on   Export to Word button on
        the toolbar. User may select the Object Template that will be applied on the exported document.
        - User selects single or multiple objects in Active workspace client and clicks on Export to Word button on the
        toolbar. If the user selects "Include attachments"option, then a zip file will be exported that includes the
        Word document along with all the attachments associated with the selected object.
        
        """
        return cls.execute_soa_method(
            method_name='exportAsync2',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=None,
        )

    @classmethod
    def exportToApplication3(cls, input: List[ExportToApplicationInputData4]) -> ExportToApplicationResponse2:
        """
        This operation exports  the selected occurrences in the structure to an application such as MSWord and MSExcel.
        The behavior of export operation is controlled by applicationFormat and objectTemplateInputs arguments
        specified in ExportToApplicationInput4 structure. If user selects the "include attachments" option, then this
        operation can export a .zip file that will include all the attachments associated with the selected object
        along with the document. The objects may be checked out during the export.
        
        Following are supported application formats from input.
        
        - MSExcelLive - This mode provides the capability of editing of properties of  underlying ItemRevision  in
        MSExcel and save the changes to Teamcenter. The generated sheet allows user to login and change the properties
        from MSExcel application. The changes will be saved to Teamcenter when user exits out of the cell in MSExcel
        application.  Teamcenter extensions for Microsoft Office should be installed on the client machine to support
        this application format.
        - MSWordXMLLive - This mode provides the capability of structural editing of SpecElementRevisions in MSWord.
        Users can edit the rich text of Requirement and perform structural changes to the exported Requirement(s) in
        the word document. Office Client should be installed on the client machine to support this application format.
        - MSExcel - This mode provides the capability of exporting objects to static MSExcel application. 
        - MSExcelReimport- This mode provides the capability of exporting objects to MSExcel application for reimport
        purpose. The generated sheet can be used to for add/remove structure elements or to update the properties on
        the SpecElement revision. The generated sheet should be reimported back to  Teamcenter. Office Client should be
        installed on the client machine to support this application format.
        - MSWordCompare - This mode provides the capability of comparing the contents of source and target objects
        using MSWord application.  The comparison document is generated asynchonously in the background and user is
        notified when the comparison document is generated. Dispatcher and notification services should be running to
        support this application format.
        - HTML- This mode provides the capability of fetching the HTML contents of input SpecElement objects.
        
        
        
        Use cases:
        - User loads a structure in ACE (Active Content Experience) panel. The ACE panel is used for smart navigation
        of occurrences in a structure. User selects an occurrence and clicks on the Export To Word or Export to Excel
        button in the toolbar. This action will allow users to select a template that will be used for export purpose.
        If user has selected a single occurrence and clicks on export, then the occurrence and all its children are
        exported to MSWord or MSExcel. If user has selected multiple occurrences and clicks on export, then the
        selected occurrences are exported to MSWord or MSExcel.
        - User selects single or multiple objects in Active Workspace Client. User clicks on the Export to Excel button
        in the toolbar. User can select View, Edit or Reimport option. Depending upon the option selected, a static
        MSExcel report or a Live MSExcel sheet or a sheet is generated that can be reimported back to Teamcenter. 
        - User selects single or multiple objects in Active Workspace Client and sets them as source objects to
        compare. Similarly user selects single or multiple target objects to compare. User clicks on the Export to Word
        button in the toolbar along with the checkbox for compare contents. User selects the compare contents checkbox
        that allows user to compare the MSWord contents of source and target objects. User is notified when the
        comparison document is generated and attached to a dataset.
        - User selects a SpecElement created from Active Workspace client and clicks on the Viewer tab. The HTML
        contents of the selected SpecElement is shown in the Viewer tab.
        - User selects an  ItemRevision and clicks on the Export To Word or Export to Excel button on the toolbar. If
        user selects "Check out" option then selected ItemRevision and its children will be checked out and exported to
        MSWord or MSExcel.
        - User selects single or multiple objects in Active Workspace Client, and clicks on   Export to Word button on
        the toolbar. User may select the Object Template that will be applied on the exported document.
        - User selects single or multiple objects in Active workspace client and clicks on Export to Word button on the
        toolbar. If user selects "Include attachments"option, then a zip file will be exported that includes the Word
        document along with all the attachments associated with the selected object.
        
        """
        return cls.execute_soa_method(
            method_name='exportToApplication3',
            library='Internal-AWS2',
            service_date='2017_06',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=ExportToApplicationResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateGroupInput(TcBaseObj):
    """
    The structure represents all of the data necessary to create interchangeable groups.
    
    :var objectType: Type of the Interchangeable Group to be created, Substitute Group or Alternate Group.
    :var props: A map of Interchangeable Group properties names and initial values pairs (string, string). Multi-valued
    properties are represented with a comma separated string.
    :var sourceObjs: A list of occurences (Occurrence level) or a list of Item/ItemRevision objects (Global level) that
    needs to be all replaced together as a group by an interchangeable group.
    :var interChangeableObjs: A list of interchangeable parts to be associcated as an Interchangeable Group.
    """
    objectType: str = ''
    props: PropNameValuesMap = None
    sourceObjs: List[BusinessObject] = ()
    interChangeableObjs: List[WorkspaceObject] = ()


"""
Interchangeable Group properties and values. This map is of property name (as key) and property values (as value) in string format. Each value is a list of strings to support both single valued and multi valued properties of types. The calling client is responsible for converting the different property types (like integer, double, date, etc) to a string using the appropriate to< type >String function (e.g. toIntString and toDateString) in the client framework's Property class.
"""
PropNameValuesMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from tcsoa.gen.Classification._2007_01.Classification import GetChildrenResponse, SearchByInstanceIdResponse, GetPartFamilyTemplatesResponse, GetKeyLOVsResponse, GetClassificationObjectsResponse, GetFileIdCriteria, ClassificationObject, SearchForClassesCriteria, GetClassDescriptionsResponse, FindClassificationObjectsResponse, GetParentsResponse, GetAttributesForClassesResponse, SearchClassAttributes, SearchResponse, FindClassifiedObjectsResponse, CreateClassificationObjectsResponse, UpdateClassificationObjectsResponse, SearchForClassesResponse, GetFileIdsResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def search(cls, searchCriteria: List[SearchClassAttributes]) -> SearchResponse:
        """
        Finds all the classification objects based on classification class IDs, classification attribute ID and an
        expression for classification attribute value. A classification object is also called an ICO.
        
        Use cases:
        User needs to search for classification objects based on the class where they are classified and the value of
        classification attributes.Another related operation for searching classification objects is
        'searchByInstanceID'(), that can search for classification objects based on their IDs
        
        Exceptions:
        >The operation throws a 'ServiceException' in case of an error condition. Clients should  then retrieve the
        errors from the 'ServiceData' list of partial errors in the returned 'SearchResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='search',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'searchCriteria': searchCriteria},
            response_cls=SearchResponse,
        )

    @classmethod
    def searchByInstanceId(cls, instanceIdQueries: List[str]) -> SearchByInstanceIdResponse:
        """
        Finds all the classification objects based on their IDs. A classification object is also called an ICO. If the
        ICO classifies a workspace object, then ICO ID would be same as the workspace object ID
        
        Use cases:
        User wants to search for classification objects based on their IDs. The returned objects can then be used as
        input for operations like 'findClassifiedObjects'(), which is used to search workspace objects associated with
        the ICOs.
        Another related operation for searching classification objects is 'search'(), that can search for
        classification objects based on class ID and attribute values
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='searchByInstanceId',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'instanceIdQueries': instanceIdQueries},
            response_cls=SearchByInstanceIdResponse,
        )

    @classmethod
    def searchForClasses(cls, criteria: List[SearchForClassesCriteria]) -> SearchForClassesResponse:
        """
        Finds the classification classes based on provided search criteria and provides detailed information about
        those classes.  The user can search using a search expression on attributes of the class (Class ID, Name, Type
        etc.)  . For example, the user shall be able to search all the classes whose name begins with a particular set
        of characters and where the class ID matches certain pattern. The order of search results can also be sorted on
        various criteria.
        
        Use cases:
        The user needs to search for classification classes using a search criterion based on various attributes of a
        class. The search criterion can be based on one or more attributes
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='searchForClasses',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'criteria': criteria},
            response_cls=SearchForClassesResponse,
        )

    @classmethod
    def updateClassificationObjects(cls, clsObjs: List[ClassificationObject]) -> UpdateClassificationObjectsResponse:
        """
        Updates existing classification objects. A classification object is also called ICO. Values of various ICO
        attributes can be modified
        
        Use cases:
        User wants to update values of the attributes for an existing classification object in Teamcenter. E.g. user
        wants to modify an integer value of a class attribute ("Length") for an existing ICO. This operation is
        typically used after creating the classification objects using 'createClassificationObjects'().
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='updateClassificationObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'clsObjs': clsObjs},
            response_cls=UpdateClassificationObjectsResponse,
        )

    @classmethod
    def createClassificationObjects(cls, clsObjs: List[ClassificationObject]) -> CreateClassificationObjectsResponse:
        """
        Creates one or more classification objects and (optionally) attach them to a workspace object, thus classifying
        it. When the Classification objects are not associated with any workspace object, they would act as standalone
        Classification objects. A classification object is also called ICO
        
        Use cases:
        User wants to classify a workspace object or create a standalone classification object (ICO) in Teamcenter.
        This operation can be combined with other operations like createItems() to create workspace object and then
        associate the workspace object to the classification object. Before creating a classification object, a
        classification class hierarchy should already be created by the classification admin user in Teamcenter. This
        hierarchy should include a storage class (a class that allows instances to be created and associated to it) for
        which the classification objects need to be created. Values of any attributes associated with classification
        objects can also be populated.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Classification object ID mapped to the error message in the 'ServiceData'
        list of partial errors in the returned 'CreateClassificationObjectsResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='createClassificationObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'clsObjs': clsObjs},
            response_cls=CreateClassificationObjectsResponse,
        )

    @classmethod
    def deleteClassificationObjects(cls, clsObjTags: List[BusinessObject]) -> ServiceData:
        """
        Deletes one or more classification objects permanently. A classification object is also called ICO. The
        classified workspace object associated with the ICO will not be deleted
        
        Use cases:
        User needs to delete classification objects. It is typically called when after creating or searching the
        classification objects, user decides that the returned objects are not needed anymore
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Classification object ID mapped to the error message in the 'ServiceData'
        list of partial errors in the returned 'DeletedClassificationObjectsResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='deleteClassificationObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'clsObjTags': clsObjTags},
            response_cls=ServiceData,
        )

    @classmethod
    def findClassificationObjects(cls, wsoIds: List[WorkspaceObject]) -> FindClassificationObjectsResponse:
        """
        Finds the classification objects associated with input workspace objects (WSO). A classification object is also
        called ICO. Each workspace object can have one or more ICOs associated with it.
        
        Use cases:
        When user need to find classification objects (ICO) based on workspace objects. Each time a workspace object is
        classified in a classification class a classification object (ICO) object is created.  After searching for all
        the classification objects corresponding to a workspace object, user can find more information about the
        classification(s) of a workspace object. The operation 'getClassificationObjects()'' 'can be used to get
        detailed information about the classification objects. After getting information on classification objects,
        user can also choose to modify or delete those using operation 'updateClassificationObjects()' or
        'deleteClassificationObjects()'
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Classification object ID mapped to the error message in the 'ServiceData'
        list of partial errors in the returned 'CreateClassificationObjectsResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='findClassificationObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'wsoIds': wsoIds},
            response_cls=FindClassificationObjectsResponse,
        )

    @classmethod
    def findClassifiedObjects(cls, icoTags: List[BusinessObject]) -> FindClassifiedObjectsResponse:
        """
        Finds the workspace objects (WSO) associated with input Teamcenter classification objects. A classification
        object is also called ICO. Each ICO can have only one workspace object associated with it.
        
        Use cases:
        When user need to find workspace objects based on classification objects (ICO) that were created when workspace
        objects were classified.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Classification object ID mapped to the error message in the 'ServiceData'
        list of partial errors in the returned 'FindClassifiedObjectsResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='findClassifiedObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'icoTags': icoTags},
            response_cls=FindClassifiedObjectsResponse,
        )

    @classmethod
    def getAttributesForClasses(cls, classIds: List[str]) -> GetAttributesForClassesResponse:
        """
        Provides information on class attributes for the classification classes based on input classification class
        ids. Detailed information about class attributes is provided & includes class attribute name, description,
        format, unit system, minimum/maximum value & configuration set
        
        Use cases:
        When user wants to view details of all class attributes associated with a classification class. This operation
        is similar to getAttributesForClasses2(), but provides information in a slightly different format. Typically,
        the information about class attributes is used to determine which classification class a workspace object shall
        be classified into
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.
        In all other cases failures will be returned with the Class ID mapped to the error message in the 'ServiceData'
        list of partial errors of the returned 'GetAttributesForClassesResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='getAttributesForClasses',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'classIds': classIds},
            response_cls=GetAttributesForClassesResponse,
        )

    @classmethod
    def getChildren(cls, groupOrClassIds: List[str]) -> GetChildrenResponse:
        """
        Gets the information about immediate children in classification hierarchy for given group or class
        identifier(s).
        Returns a 'GetChildrenResponse' structure containing:
        - Retrieved child classes in the 'ServiceData' list of plain objects
        - Any failures with Class ID mapped to the error message in the 'ServiceData' list of partial errors.
        
        
        
        Use cases:
        User wants to get details of all groups or classes that lie under a particular group or class in a
        classification class hierarchy.
        
        Returns a 'GetChildrenResponse' structure containing:
        - Retrieved child classes in the 'ServiceData' list of plain objects
        - Any failures with Class ID mapped to the error message in the 'ServiceData' list of partial errors.
        
        
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.
        In all other cases failures will be returned with the Class ID mapped to the error message in the 'ServiceData'
        list of partial errors of the returned 'GetChildrenResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='getChildren',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'groupOrClassIds': groupOrClassIds},
            response_cls=GetChildrenResponse,
        )

    @classmethod
    def getClassDescriptions(cls, classIds: List[str]) -> GetClassDescriptionsResponse:
        """
        Gets detailed information about a classification class based on classification class ID. This information
        includes class type, parent, name, description, unit system and user data associated with the class.  It also
        includes a count of children, number of classification views & number of instances of classification objects
        associated with the classification class. Information can also be obtained on any documents such as images &
        icons attached to the classification class.
        
        Use cases:
        When user need details of classification classes. These details can help user decide whether to classify a
        workspace object in particular classification classes.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getClassDescriptions',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'classIds': classIds},
            response_cls=GetClassDescriptionsResponse,
        )

    @classmethod
    def getClassificationObjects(cls, clsObjTags: List[BusinessObject]) -> GetClassificationObjectsResponse:
        """
        Looks for specified classification objects. If they are found, then detailed information about those objects is
        provided. A classification object is also called ICO
        
        Use cases:
        When user need to find an existing classification object to either view or update its details. It can be
        followed by operations like 'updateClassificationObjects()' or 'deleteClassificationObjects()' to update or
        delete the classification objects. 
        The operation 'findClassificationObjects()' can be used to get the list of classification objects, associated
        with workspace objects. Then, this operation 'getClassificationObjects()' is used to get the detailed
        information on the classification objects.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.In all other cases
        failures will be returned with the Classification object ID mapped to the error message in the 'ServiceData'
        list of partial errors in the returned 'GetClassificationObjectsResponse' structure.
        """
        return cls.execute_soa_method(
            method_name='getClassificationObjects',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'clsObjTags': clsObjTags},
            response_cls=GetClassificationObjectsResponse,
        )

    @classmethod
    def getFileIds(cls, criteria: List[GetFileIdCriteria]) -> GetFileIdsResponse:
        """
        Gets the file information from any dataset that is associated with workspace object(s). The dataset type can be
        specified along with the relation used when it is attached to a workspace object. Information corresponding to
        a particular file inside a dataset can be retrieved.
        
        Use cases:
        User wants to get information about files inside a dataset that is associated with workspace objects (WSO).
        Typically it will be used to get and view the  image or icon files associated with datasets attached to
        workspace objects.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getFileIds',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'criteria': criteria},
            response_cls=GetFileIdsResponse,
        )

    @classmethod
    def getKeyLOVs(cls, keyLOVIds: List[str]) -> GetKeyLOVsResponse:
        """
        Gets the information for classification key-LOVs  based on given ID(s). Information such as key-LOV's name,
        display options, and key and value entries can be obtained. A key-LOV is a list of values used in
        classification. The key-LOVs are used to define one or more values that can be set for classification
        dictionary attributes
        
        Typical format of a key-LOV -
        
            <key-LOV ID>:<key-LOV Name>
            <Key10>:<Value10>
            <Key20>:<Value20>
        
        Example of a key-LOV
        
        - 33381:Design Categories
            Des1:Bearing
            Des2:Bracket
            Des3:Frame
            Des4:LeadBox
        
        
        Use cases:
        User wants to retrieve the information for an existing key-LOV using the key-LOV's unique identifier. The
        operation is similar to 'getKeyLOVs2'(). But 'getKeyLOVs2'()provides more detailed information about any
        key-LOVs .
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception. In all other cases
        failures will be returned with the Key-LOV ID mapped to the error message in the 'ServiceData' list of partial
        errors of the 'GetKeyLOVsResponse' return structure.
        """
        return cls.execute_soa_method(
            method_name='getKeyLOVs',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'keyLOVIds': keyLOVIds},
            response_cls=GetKeyLOVsResponse,
        )

    @classmethod
    def getParents(cls, childIds: List[str]) -> GetParentsResponse:
        """
        Gets the classification class ID(s) of all parent classes in a hierarchy, based on given classification class
        ID. The parent class IDs are sorted as immediate parent first, toplevel parent last.
        
        Use cases:
        User needs to determine all the parent classes for any given class in a classification hierarchy.  If user
        needs to get the children of the given class ID, then 'getChildren'() operation shall be used.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' for an unknown type of exception.
        In all other cases failures will be returned with the Class ID mapped to the error message in the 'ServiceData'
        list of partial errors of 'GetParentsResponse' return structure.
        """
        return cls.execute_soa_method(
            method_name='getParents',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'childIds': childIds},
            response_cls=GetParentsResponse,
        )

    @classmethod
    def getPartFamilyTemplates(cls, clsClassIds: List[str]) -> GetPartFamilyTemplatesResponse:
        """
        Finds the information for part family templates (PFT) based on the classification class IDs. Part family
        templates can be used to define geometry and certain properties of the geometry as variable properties. They
        can be attached to a classification class. For any classification class, user can find out the associated part
        family templates and their information.
        
        Use cases:
        While using graphics builder, users often require information about the part family template attached to the
        classification classes.  Graphics builder is a program used by classification administration that communicates
        with the Teamcenter server to generate graphics. The graphics builder uses NX libraries.
        
        Exceptions:
        >The operation will only throw a 'ServiceException' in cases when an error condition cannot be handled and an
        appropriate error message could not be added to the list of partial errors in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='getPartFamilyTemplates',
            library='Classification',
            service_date='2007_01',
            service_name='Classification',
            params={'clsClassIds': clsClassIds},
            response_cls=GetPartFamilyTemplatesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2018_06.StructureManagement import AssemblyConfigurationResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def writeAssemblyConfigurationDetails(cls, bomWindow: BusinessObject, optionSetName: str) -> AssemblyConfigurationResponse:
        """
        This operation gets the fully unpacked product structure for the input configured structure. It takes as input
        a bomWindow object of a structure that is already configured, in say, the Teamcenter Rich client and  a TIE
        export option set name. The fully qualified product structure that in the output can be used by, say, a CAD
        client to open the structure with the exact same configuration.
        This operation allows a user to configure a product structure in the Teamcenter Rich Client (RAC) and then open
        it in a CAD or other client application. That hand-off includes a fully qualified product structure that the
        CAD application would use to construct the product in the exact same configuration as that in the RAC.
        This operation uses the TIE export process to get the product structure data to write to the file. The
        structure data written will be the fully unpacked structure.
        The export option set will control the level and scope of the structure and related objects to traverse.
        Individual CAD integrations should define or create an option set that includes the CAD data types.
        
        Use cases:
        A user configures an assembly in a particular way, using Revision Rule, Variants, Effectivity, etc in the
        Strcuture Manager application in RAC. After setting up all the configuration details, the user wishes to open
        the assembly in a CAD application.  The user expectation is that the assembly as opened in the CAD application
        will mirror the product structure in the Structure Manager.
        
        To make this happen, the CAD application should add a custom "Open in CAD" command in the Structure Manager
        application. The command will call this SOA operation with the selected BOM Window as the input. The TIE export
        option to be used will be CAD specific.
        This operation will then write the structure data to an xml file in the transient volume and return the file
        read ticket. The file can be downloaded by using the regular file download ITKs/SOAs. 
        The CAD application will download the file and read the structure details and load/open the structure
        accordingly. 
        These are the various use cases used to configure Product Structure in Structure Manager and used to write data
        in TCXML file.
        """
        return cls.execute_soa_method(
            method_name='writeAssemblyConfigurationDetails',
            library='Cad',
            service_date='2018_06',
            service_name='StructureManagement',
            params={'bomWindow': bomWindow, 'optionSetName': optionSetName},
            response_cls=AssemblyConfigurationResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cae._2012_02.StructureManagement import ExecuteRuleResponse2
from tcsoa.gen.BusinessObjects import Snapshot, StructureMapRevision, RevisionRule, VariantRule, ItemRevision
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def executeDatamap(cls, rootIR: ItemRevision, snapshotFolder: Snapshot, revRule: RevisionRule, variantRule: VariantRule, domain: str) -> ExecuteRuleResponse2:
        """
        This operation creates an output BOM structure given the root ItemRevision of the root BOMLine of an input BOM
        structure along with its RevisionRule and the VariantRule. A Snapshot folder of the input BOM structure along
        with the VariantRule can also be provided as an input. The output BOM structure is determined by the XSLT-based
        Data Map rules executed against the input BOM structure. Data Map syntax is in compliance with the schema
        defined in tcsim_xslWithNode.xsd, located in TC_DATA.
        
        Data Map rules define the mapping between an input item type and its resulting output item type. Data Map rules
        are defined for an entire site and are stored in the datamapping.xml file located in TC_DATA. The name of the
        datampping file is defined by the site preference CAE_dataMapping_file.
        
        The Data Map rules can be configured for various domains defined as LOV objects under StructureMap Domains in
        BMIDE. To configure the domains, in the Extensions view in BMIDE, open LOV->StructureMap Domains and add
        additional domain values. The domain to be used for applying Data Map rules can also be provided as an input.
        
        To use this operation, a well-defined datamapping.xml is required in TC_DATA and the user should have either a
        simulation_author or rtt_author license.
        
        
        Use cases:
        Use Case 1: Create an output structure given a top BOMLine of the input structure along with its configuration
        Given an input root BOMLine of a BOM structure, along with its RevisionRule and VariantRule, the user can apply
        Data Map rule to the BOM structure and generate a corresponding output BOM structure. The output BOM structure
        would consist of BOMLine occurrences of ItemRevision objects as defined in the datamapping.xml file. The user
        can review the actions executed with the process log returned with the BOMViewRevision. An email notification
        containing the activity log would be sent to the current user if the session option for email notification is
        set to true.
        
        Use Case 2: Create an output structure given a Snapshot folder of the input structure along with the variant
        rule
        Given a Snapshot folder of the input BOM structure and its VariantRule, the user can apply Data Map rules to
        the BOM structure and generate a corresponding output BOM structure. The output BOM structure would consist of
        BOMLine occurrences of ItemRevision objects as defined in the datamapping.xml file. The user can review the
        actions executed with the process log returned with the BOMViewRevision. An email notification containing the
        activity log would be sent to the current user if the session option for email notification is set to true.
        """
        return cls.execute_soa_method(
            method_name='executeDatamap',
            library='Cae',
            service_date='2012_02',
            service_name='StructureManagement',
            params={'rootIR': rootIR, 'snapshotFolder': snapshotFolder, 'revRule': revRule, 'variantRule': variantRule, 'domain': domain},
            response_cls=ExecuteRuleResponse2,
        )

    @classmethod
    def executeStructureMap(cls, rootIR: ItemRevision, snapshotFolder: Snapshot, revRule: RevisionRule, variantRule: VariantRule, structureMapIR: StructureMapRevision) -> ExecuteRuleResponse2:
        """
        This operation creates an output BOM structure given the root ItemRevision of the root BOMLine of an input BOM
        structure along with its RevisionRule and the VariantRule. A Snapshot folder of the input BOM structure along
        with the VariantRule can also be provided as an input. The output BOM structure is determined by a combination
        of XSLT-based Data Map and StructureMap rules executed against the input BOM structure. Data Map/StructureMap
        syntax is in compliance with the schema defined in tcsim_xslWithNode.xsd, located in TC_DATA.
        
        Data Map rules define the mapping between an input item type and its resulting output item type. Data Map rules
        are defined for an entire site and are stored in the datamapping.xml file located in TC_DATA. The name of the
        data mapping file is defined by the site preference CAE_dataMapping_file.
        
        StructureMap rules tailor the output BOM Structure. There are several rule types:
        - Filter - Removes input BOM lines (and their children) from Data Map evaluation.
        - Include - Inserts item revisions in either the input or output BOM structure as required.
        - Reuse - Retrieve existing item revision to be used in the output structure.
        - Create Collector - Reorganization rule that creates "container" item revisions to move BOMLine objects and
        sub-assemblies around.
        - Move to Collector - Reorganizational rule that moves BOMLine objects and sub-assemblies to collector
        components.
        - Collapse Single Component Assembly - Identifying sub-assemblies with single child component, elevating the
        child component to the parent sub-assembly and removing the parent sub-assembly.
        - Remove Empty Assembly - Identifying sub-assemblies with no child components and removing the empty
        sub-assembly.
        - Skip - Skips the BOMLine but still process its children.
        
        
        
        StructureMap rules are stored an XML named reference in CAEStructureMap dataset attached to a
        StructureMapRevision. StructureMap rules are created with Simulation Process Management CAE Structure Designer.
        
        To use this operation, a well-defined datamapping.xml is required in TC_DATA and a StructureMapRevision with an
        attached CAEStructureMap dataset must exist and the user should have either a simulation_author or rtt_author
        license.
        
        Use cases:
        Use Case 1: 
        Given an input root BOMLine of a BOM structure, along with its RevisionRule and VariantRule, the user can apply
        a StructureMap rule to the BOM structure and generate a corresponding output BOM structure. The output BOM
        structure would consist of BOMLine occurrences of ItemRevision objects as defined in the datamapping.xml file
        and would be organized by the StructureMap rules defined in the CAEStructureMap dataset attached to the
        StructureMapRevision. The user can review the actions executed with the process log returned with the
        BOMViewRevision. An email notification containing the activity log would be sent to the current user if the
        session option for email notification is set to true.
        
        Use Case 2: 
        Given a Snapshot folder of the input BOM structure and its VariantRule, the user can apply a StructureMap rule
        to the BOM structure and generate a corresponding output BOM structure. The output BOM structure would consist
        of BOMLine occurrences of ItemRevision objects as defined in the datamapping.xml file and would be organized by
        the StructureMap rules defined in the CAEStructureMap dataset attached to the StructureMapRevision. The user
        can review the actions executed with the process log returned with the BOMViewRevision. An email notification
        containing the activity log would be sent to the current user if the session option for email notification is
        set to true.
        """
        return cls.execute_soa_method(
            method_name='executeStructureMap',
            library='Cae',
            service_date='2012_02',
            service_name='StructureManagement',
            params={'rootIR': rootIR, 'snapshotFolder': snapshotFolder, 'revRule': revRule, 'variantRule': variantRule, 'structureMapIR': structureMapIR},
            response_cls=ExecuteRuleResponse2,
        )



from __future__ import annotations

from tcsoa.gen.RelationshipViewer._2014_11.NetworkEngine import GraphTypeListResponse
from tcsoa.utils import TcService


class NetworkEngineService(TcService):

    @classmethod
    def getViews3(cls, key: str) -> GraphTypeListResponse:
        """
        This operation provides a list of the available graph view types identified by the input key. A graph view is a
        set of configuration that can be used for network expansion. For example: localized view name and category
        types which classify a list of supported Teamcenter Business Object types, legend color of each category types.
        The view list is role based, different role may get different view list.
        """
        return cls.execute_soa_method(
            method_name='getViews3',
            library='RelationshipViewer',
            service_date='2014_11',
            service_name='NetworkEngine',
            params={'key': key},
            response_cls=GraphTypeListResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0AbsFamilyGroup, Cfg0AbsFamily, BusinessObject, Cfg0ConfiguratorPerspective, Cfg0AbsConfiguratorWSO
from tcsoa.gen.Internal.Configurator._2015_10.ConfiguratorManagement import Violation, BusinessObjectConfigExpression
from tcsoa.gen.Internal.Configurator._2017_11.ConfiguratorManagement import ConfigurationProfile
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class Variability(TcBaseObj):
    """
    The structure represents output variability information such as all configured model families, models, family
    groups, families and values.
    
    :var models: List of model families and their models. The list will not be in any specific order.
    :var groupedVariability: List of groups and their member families with values. The list will not be in any specific
    order.
    :var ungroupedVariability: List of ungrouped families and their vaues. The list will not be in any specific order.
    """
    models: List[FamilyInfo] = ()
    groupedVariability: List[GroupInfo] = ()
    ungroupedVariability: List[FamilyInfo] = ()


@dataclass
class ViolationList(TcBaseObj):
    """
    Structure contains violated rules along with the message and severity associated with rule violation. It is
    structure which represent Independent set of constraint rules which are responsible for conflict.
    
    :var statusCode: Status code.
    :var statusMessage: Status message
    :var violations: Set of violated rules which together are responsible for conflict.
    """
    statusCode: int = 0
    statusMessage: str = ''
    violations: List[Violation] = ()


@dataclass
class CellReference(TcBaseObj):
    """
    Represents address of a cell within an Expression Matrix.
    
    :var expressionToValidateClientID: Client ID of an Expression to validate. Client IDs are generated by callers of
    the this Operation. Callers of the Operation use it to correlate between Operation input and Operation output.
    :var scopeExpressionClientID: Client ID of a Scope Expression. Client IDs are generated by callers of the this
    Operation. Callers of the Operation use it to correlate between Operation input and Operation output.
    """
    expressionToValidateClientID: str = ''
    scopeExpressionClientID: str = ''


@dataclass
class CellValidationResult(TcBaseObj):
    """
    Structure containing list of structure instances which in turn contains pair of identifiers for expression to
    validate and scope expression along with validation operation output. It contains a one entry for each pair of
    expressions from input structure expressionMatrix and identified with list of pair of integers matrixSubset.
    
    :var cellClientIDs: Pair of identifiers where first identifier is of Expression to validate and second identifier
    is of scope Expression.
    :var verdictStruct: Validation operation output and Expression identifier information per processed Expression
    Matrix cell.
    """
    cellClientIDs: CellReference = None
    verdictStruct: ValidateProductConditionOutptStruct = None


@dataclass
class ConfigurationSessionInfoOutput(TcBaseObj):
    """
    The structure for retrieving a Product Configurator configuration session information. A Product Configurator
    session can be constructed using the session information.
    
    :var sessionConfigObject: The session configurable object of type VariantRule or Cfg0AbsConfiguratorCriteria for
    which the product information and Configuration Profile is retrieved.
    :var perspective: The Cfg0ConfiguratorPerspective instance which provides the product information used to select
    the features and constraints. The product information consists of a Cfg0ConfContext, RevisionRule, Effectivity,
    Rule Date.
    :var configProfile: The Configuration Profile information which contains the parameters governing the constraint
    evaluation behavior in a Product Configurator configuration session.
    """
    sessionConfigObject: BusinessObject = None
    perspective: Cfg0ConfiguratorPerspective = None
    configProfile: ConfigurationProfile = None


@dataclass
class ExpressionMatrix(TcBaseObj):
    """
    It is structure which pairs list of BusinessObjectConfigExpressions.
    
    :var expressionsToValidate: List of Expression(s) to validate.
    :var scopeExpressions: List of scope Expression(s) against which expressions from list expressionsToValidate to
    validate.
    """
    expressionsToValidate: List[BusinessObjectConfigExpression] = ()
    scopeExpressions: List[BusinessObjectConfigExpression] = ()


@dataclass
class FamilyInfo(TcBaseObj):
    """
    The FamilyInfo is a structure which represents family and the values belongs to it. Here family can be model family
    or option family or dynamic family and value can be model or option value or standalone feature.
    
    :var family: The Cfg0AbsFamily object.
    :var values: List of Cfg0AbsConfiguratorWSO objects, the objects can be objects of type, Cfg0AbsValue or
    Cfg0AbsFeature. The list of values will not be in any specific order.
    """
    family: Cfg0AbsFamily = None
    values: List[Cfg0AbsConfiguratorWSO] = ()


@dataclass
class GetConfigurationSessionInfoResponse(TcBaseObj):
    """
    The response structure for retrieving a number of Product Configurator configuration session information
    structures. A session information retrieved from a session configurable object is returned with the same object.
    The session information is retrieved as a combination of Cfg0ConfiguratorPerspective representing the product
    information and Configuration Profile which represents the parameters for constraint evaluation. The session
    information can be used to recreate a Product Configurator session. The service data returns partial error
    encountered in retrieving the session information from an input object against the index of the object in input
    list.
    
    :var configSessionInfos: A list of  session information retrieved from a session configurable object is returned
    with the same object. The session information is retrieved as a combination of Cfg0ConfiguratorPerspective
    representing the product information and Configuration Profile which represents the parameters for constraint
    evaluation. The session information can be used to recreate a Product Configurator session.
    :var serviceData: The service data returns partial error encountered in retrieving the session information from an
    input object against the index of the object in input list.
    """
    configSessionInfos: List[ConfigurationSessionInfoOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetVariabilityResponse(TcBaseObj):
    """
    The response structure is returned by the operation getVariablility. The list of variability in the output response
    will be populated in the sequence of input list of configurator perspectives in the request. If there is any
    failure, the service data will be populated with partial error at the same index of input configurator perspective.
    The variability can be all configured model families, models, groups, families and values.
    
    :var serviceData: The service data for errors and returned objects.
    :var variabilityList: List of model families and their models, list of groups and their families with values and
    ungrouped families with their values.
    """
    serviceData: ServiceData = None
    variabilityList: List[Variability] = ()


@dataclass
class GroupInfo(TcBaseObj):
    """
    The GroupInfo is a structure which represents configured group and the families belong to it.
    
    :var familyGroup: The Cfg0AbsFamilyGroup object.
    :var families: List of  familyGroup&rsquo;s member families with their values. The list of families will not be in
    any specific order.
    """
    familyGroup: Cfg0AbsFamilyGroup = None
    families: List[FamilyInfo] = ()


@dataclass
class MatrixCell(TcBaseObj):
    """
    It is structure which represents a pair of integer.
    
    :var rowIndex: Integer type object.
    :var columnIndex: Integer type object.
    """
    rowIndex: int = 0
    columnIndex: int = 0


@dataclass
class ValidateProductConditionOutptStruct(TcBaseObj):
    """
    Validation operation output information.
    
    :var criteriaStatus: The status of the configuration. 
    Valid values are: validAndComplete, validAndInComplete or inValid. 
    validAndComplete: All mandatory families in configuration have value selections. 
    validAndInComplete: Not all mandatory families in configuration have value selections. 
    inValid: There are some violations .
    :var conflicts: List of conflicts that was identified for a given cell in the validation matrix. A conflict is
    represented by a list of constraint rule violations
    """
    criteriaStatus: str = ''
    conflicts: List[ViolationList] = ()


@dataclass
class ValidateProductConditionResponse(TcBaseObj):
    """
    The response structure for the operation validateProductConditions.
    
    :var serviceData: The service data for errors and returned objects.
    :var validationResults: Validation operation output information along with identifiers of input Expressions for
    input and output correlation.
    """
    serviceData: ServiceData = None
    validationResults: List[CellValidationResult] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import TC_Project, User
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProjectInfo(TcBaseObj):
    """
    This structure holds the TC_Project and a flag indicating the user's membership type.
    
    :var project: The TC_Project object.
    :var isUserPrivileged: Flag indicating if the given  user is a privleged  member.
    """
    project: TC_Project = None
    isUserPrivileged: bool = False


@dataclass
class UserProjectsInfo(TcBaseObj):
    """
    This structure holds the projects for one of the given users.
    
    :var user: The User object of Teamcenter.
    :var activeProjectsOnly: Flag to indicate the status of projects  returned True indicates only active projects are
    returned. False indicates both active and inactive projects are returned.
    :var privilegedProjectsOnly: Flag to indicate if returned projects  are privleged or not.True indicates projects
    are privileged, false indicates all projects where the user is a member of regardless of privilege in the projects.
    :var programsOnly: Flag Indicating if user wants to get program only pojects of the user, false indicates returning
    all projects where regardless of status program only  value of TC_Project.
    :var clientId: An id associated with a client.
    :var projectsInfo: A list of ProjectInfo structure.
    """
    user: User = None
    activeProjectsOnly: bool = False
    privilegedProjectsOnly: bool = False
    programsOnly: bool = False
    clientId: str = ''
    projectsInfo: List[ProjectInfo] = ()


@dataclass
class UserProjectsInfoInput(TcBaseObj):
    """
    This structure holds the User object and criteria to find the user projects.
    
    :var user: The User object of Teamcenter.
    :var activeProjectsOnly: Flag Indicating if user wants to get active projects or 
    not.True indicates return only active projects of the user, false indicates return both active and inactive
    projects of the user.
    :var privilegedProjectsOnly: Flag Indicating if user wants to get privleged projects  
    or not.True indicates return privileged projects of the user, false indicates return all projects where the user is
    a member of regardless of status.
    :var programsOnly: Flag Indicating if user wants to get program only projects of the user, false indicates return
    all projects regardless of status program only.
    :var clientId: An id associated with a client.
    """
    user: User = None
    activeProjectsOnly: bool = False
    privilegedProjectsOnly: bool = False
    programsOnly: bool = False
    clientId: str = ''


@dataclass
class UserProjectsInfoResponse(TcBaseObj):
    """
    This structure holds the projects for all the given users.
    
    :var userProjectInfos: List of UserProjectsInfo structures one for each given user..
    :var serviceData: A  standard ServiceData.
    """
    userProjectInfos: List[UserProjectsInfo] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Diagramming._2011_06.DiagramManagement import CreateDiagramResponse, CreateOrUpdateTemplateResponse, OpenDiagramResponse, GetDiagramMembersResponse, SaveDiagramInputInfo, GetDiagramMembersInputInfo, CreateDiagramInputInfo, CreateOrUpdateTemplateInputInfo, OpenDiagramInputInfo, SaveDiagramResponse
from typing import List
from tcsoa.utils import TcService


class DiagramManagementService(TcService):

    @classmethod
    def openDiagram(cls, inputData: List[OpenDiagramInputInfo]) -> OpenDiagramResponse:
        """
        This operation opens a Visio Diagram created for a Teamcenter object (WorkspaceObject/ Item Revision/ Folder/
        BOMLine). The objects and relations shown on the diagram are gathered by executing the Transfer Mode and
        Relation Rule specified on the diagram template. Relation Rule specifies the relations to be shown between the
        objects.
        Only those objects appear on the diagram for which there is a Visio Shape in the stencil and there is mapping
        defined between the Shape and the Teamcenter object in the mapping xml file. Mapping file has details about how
        to define the mappings.
        If the contents of the root object of the diagram are changed, for instance, a BOM structure changes or the
        contents of the folder are modified, then the diagram will show only the current information. This is because
        every time a diagram is opened, Transfer mode and Relation Rules are applied.
        
        Use cases:
        You can select a Teamcenter object and launch the open diagram dialog which shows the list of all diagrams
        created for that object. You can select a diagram to open. A Visio diagram is opened on the RAC. The content of
        the diagram is decided by the Transfer mode and the Relation Rule.
        """
        return cls.execute_soa_method(
            method_name='openDiagram',
            library='Diagramming',
            service_date='2011_06',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=OpenDiagramResponse,
        )

    @classmethod
    def saveDiagram(cls, inputData: List[SaveDiagramInputInfo]) -> SaveDiagramResponse:
        """
        This operation saves Visio Diagram(s) created for a Teamcenter object (WorkspaceObject/ Item Revision/ Folder/
        BOMLine). When a Visio diagram is saved, a skeleton file (.vdx) is created and stored as a dataset to hold all
        the non Teamcenter information, like notes if any. A preview image of the diagram content is also stored as an
        image data set. These Dataset objects are attached to the diagram object.
        For the Teamcenter business objects appearing on the diagram, Fnd0ShapeRelation objects are created with the
        diagram object to hold the position information so that the shapes appear in their original position and format
        next time the diagram is opened.
        
        Use cases:
        You can create or open an existing diagram and make changes to it. You can then save the diagram.
        """
        return cls.execute_soa_method(
            method_name='saveDiagram',
            library='Diagramming',
            service_date='2011_06',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=SaveDiagramResponse,
        )

    @classmethod
    def createDiagram(cls, inputData: List[CreateDiagramInputInfo]) -> CreateDiagramResponse:
        """
        This operation creates a Visio Diagram for a Teamcenter object (WorkspaceObject/ ItemRevision/ Folder/ BOMLine)
        using a Diagram Template. The objects and relations shown on the diagram are gathered by executing the Transfer
        Mode and Relation Rule specified on the diagram template. Relation Rule specifies the relations to be shown
        between the objects.
        Only those objects appear on the diagram for which there is a Visio Shape in the stencil and there is mapping
        defined between the Shape and the Teamcenter object in the mapping xml file. Mapping file has details about how
        to define the mappings.
        While creating a diagram, if user selects the "Open Diagram" option, the diagram object is created on the
        server and all the necessary information is returned in the response which is utilized to open the Visio
        diagram. If user does not select the option, then just the diagram object is created.
        
        
        Use cases:
        You can select a Teamcenter object in RAC and create a Visio diagram for the same by selecting a Diagram Domain
        and a Diagram Template.
        """
        return cls.execute_soa_method(
            method_name='createDiagram',
            library='Diagramming',
            service_date='2011_06',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=CreateDiagramResponse,
        )

    @classmethod
    def createOrUpdateTemplate(cls, inputData: CreateOrUpdateTemplateInputInfo) -> CreateOrUpdateTemplateResponse:
        """
        createOrUpdateTemplate provides the ability to either create a new diagram template or modify an existing
        diagram template.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateTemplate',
            library='Diagramming',
            service_date='2011_06',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=CreateOrUpdateTemplateResponse,
        )

    @classmethod
    def getDiagramMembers(cls, inputData: List[GetDiagramMembersInputInfo]) -> GetDiagramMembersResponse:
        """
        This operation retrieves the specific types of members of the diagram based on its membership types. Following
        are the types of members apart from the ones that appear on the diagram as a result of transfer mode traversal
        and relationship rules. 
        - User Added - The objects which are added to the diagram by the user. E.g. an Item object which is copied from
        Teamcenter and then pasted on to the diagram.
        - User Omitted - The objects which are deleted by the user from the diagram. E.g. user can remove an object
        from the diagram only. The object is removed from the diagram and not from database. It can be retrieved and
        again shown on the diagram.
        
        
        
        Use cases:
        None.
        """
        return cls.execute_soa_method(
            method_name='getDiagramMembers',
            library='Diagramming',
            service_date='2011_06',
            service_name='DiagramManagement',
            params={'inputData': inputData},
            response_cls=GetDiagramMembersResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ACLNameInfo(TcBaseObj):
    """
    This structure holds the Named ACL object of requested type and its name.
    
    :var aclObject: Named ACL object.
    :var aclName: The name of the named ACL object.
    """
    aclObject: BusinessObject = None
    aclName: str = ''


@dataclass
class ACLInfoResponse(TcBaseObj):
    """
    This object contains a list of ACL objects of given type and their names.
    
    :var aclNameInfos: A list of  ACLNameInfo objects one for each  named ACL found for the given type.
    :var serviceData: Object that holds the partial errors that occur while getting the named  ACLs of given type.
    """
    aclNameInfos: List[ACLNameInfo] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Core._2018_06.LogicalObject import RootObject, ClassificationObjectInfo, MemberClassificationObject2
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetLogicalObjectInput3(TcBaseObj):
    """
    The structure holds:
    1.    A list of RootObject(s) structures which represents client id and root object instance pair.
    2.    A logical type name. 
    3.    A map containing the included logical object ID, or member ID and configuration context UID. 
    
    Above input data are used to retrieve the matched logical object instances and their configured included logical
    object instances.
    
    :var rootInstances: A list of root instance structures.
    :var loTypeName: Name of the logical object type for which the logical object instances are to be searched for. The
    logical object type name corresponds to the name of any subtype of Fnd0LogicalObject.
    :var loQueryNameValues: A map (string, string) of the Included Logical Object ID, or member ID and its
    Configuration Context UID.
    """
    rootInstances: List[RootObject] = ()
    loTypeName: str = ''
    loQueryNameValues: LOQueryNameValueMap = None


@dataclass
class GetLogicalObjectOutput3(TcBaseObj):
    """
    The structure holds the list of the retrieved logical object instances, related classification objects and included
    logical object instances.
    
    :var logicalObjectsOutput: A list of the retrieved logical object instances. The length of this member is equal to
    the length of the input rootInstances member of the input structure GetLogicalObjectInput3.
    """
    logicalObjectsOutput: List[LogicalObjectOutput3] = ()


@dataclass
class GetLogicalObjectResponse3(TcBaseObj):
    """
    The structure holds the list of the retrieved logical object instances along with classification object output
    response and partial errors.
    
    :var serviceData: Returned service data.
    :var loOutputs: A list of the retrieved logical object output response corresponding to a  logical object input.
    :var classificationObjectInfo: A map(string, ClassificationObjectInfo)  of the ICO IDs and their classification
    object info.
    """
    serviceData: ServiceData = None
    loOutputs: List[GetLogicalObjectOutput3] = ()
    classificationObjectInfo: ClassificationObjectInfoMap3 = None


@dataclass
class LogicalObjectInstance2(TcBaseObj):
    """
    The structure holds the logical object instance, its root, its members and its classification objects. The
    structure also holds the data for included logical object instances.
    
    :var loInstance: An instance of the Logical Object Type.
    :var memberClassificationObjects: A list of structures containing classification object data for the root or member
    of the Logical Object Type.
    :var includedLOInstances: The included Logical Object instances.
    """
    loInstance: BusinessObject = None
    memberClassificationObjects: List[MemberClassificationObject2] = ()
    includedLOInstances: IncludedLOInstanceMap = None


@dataclass
class LogicalObjectOutput3(TcBaseObj):
    """
    The structure contains the logical object instances for a given root object.
    
    :var root: An instance of the root object. This is same object which was provided in input RootObject structure.
    :var logicalObjectInstances: A list of logical object instances. Only one logical object instance is returned for a
    given root object in the current implementation.
    """
    root: BusinessObject = None
    logicalObjectInstances: List[LogicalObjectInstance2] = ()


"""
The map contains the included logical object ID as key and its LogicalObjectInstance2 structures as values.
"""
IncludedLOInstanceMap = Dict[str, List[LogicalObjectInstance2]]


"""
The map contains the included logical object ID, or member ID as key and Configuration Context UID as value.
"""
LOQueryNameValueMap = Dict[str, str]


"""
This structure contains the classification object ID as key and its ClassificationObjectInfo structure as values.
"""
ClassificationObjectInfoMap3 = Dict[str, ClassificationObjectInfo]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, User, POM_imc, ImanVolume, TC_Project, TC_WorkContext, Group
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetTCSessionInfoResponse(TcBaseObj):
    """
    Data structure representing the different current user's Teamcenter session information.
    
    :var serverVersion: The version of the server.
    :var transientVolRootDir: Path to the root folder of the transient volume.
    :var site: The site object for this session
    :var bypass: True if bypass is enabled.
    :var journaling: True if journaling is enabled.
    :var appJournaling: True if application journaling is enabled.
    :var secJournaling: True if sec journaling is enabled.
    :var admJournaling: True if administration journaling is enabled.
    :var privileged: True if the user is privileged.
    :var isPartBOMUsageEnabled: True if the Part BOM Usage is enabled.
    :var isSubscriptionMgrEnabled: True if the Subscription Manager is enabled.
    :var textInfos: textInfos
    :var isInV7Mode: True if the server is operating in V7 mode.
    :var extraInfo: Map of kev/value pairs (string/string).The following keys are returned:
    - TcServerIDUnique  ID of this Teamcenter server instance.
    - systemTypeType  of server, always 'Teamcenter'.
    - syslogFile  The absolute path of the system log file (.syslog) on the server host.
    - Hostname  The host name of the machine hosting the Teamcenter server process.
    - TCServerLocale The locale of the Teamcenter server.
    - currentOrganization The UID of the user's current Organization. The business object instance is in the
    ServiceData plain list
    - loginGroupOrganization  The UID of the user's login Organization. The business object instance is in the
    ServiceData plain list
    - currentChangeNotice  The UID of the user's current ChangeNotice The business object instance is in the
    ServiceData plain list
    - locationCodePref  The preferred location code.
    - displayCurrentCountryPage  True if the current country selection page is needed.
    
    
    :var serviceData: serviceData
    :var moduleNumber: This element is not to be used anymore and always returns a -1.
    :var user: The User object for this session.
    :var group: The Group object for this session.
    :var role: The Role object for this session.
    :var tcVolume: The ImanVolume object for this session.
    :var project: The Project object for this session.
    :var workContext: The WorkContext object for this session.
    """
    serverVersion: str = ''
    transientVolRootDir: str = ''
    site: POM_imc = None
    bypass: bool = False
    journaling: bool = False
    appJournaling: bool = False
    secJournaling: bool = False
    admJournaling: bool = False
    privileged: bool = False
    isPartBOMUsageEnabled: bool = False
    isSubscriptionMgrEnabled: bool = False
    textInfos: List[TextInfo] = ()
    isInV7Mode: bool = False
    extraInfo: ExtraInfo = None
    serviceData: ServiceData = None
    moduleNumber: int = 0
    user: User = None
    group: Group = None
    role: Role = None
    tcVolume: ImanVolume = None
    project: TC_Project = None
    workContext: TC_WorkContext = None


@dataclass
class MultiPreferencesResponse(TcBaseObj):
    """
    Multiple Preferences Response
    
    :var preferences: List of Returned Preferences
    :var serviceData: The successful Object ids, partial errors and failures
    """
    preferences: List[ReturnedPreferences] = ()
    serviceData: ServiceData = None


@dataclass
class ReturnedPreferences(TcBaseObj):
    """
    Info for one preference
    
    :var name: The preference name.
    :var scope: The scope of the preference, "all", "site", "user", "group", or "role".
    :var values: The values for the perference.
    """
    name: str = ''
    scope: str = ''
    values: List[str] = ()


@dataclass
class ScopedPreferenceNames(TcBaseObj):
    """
    Info for getPreferences
    
    :var scope: The scope of the preference, "all", "site", "user", "group", or "role".
    :var names: The names of the perferences.
    """
    scope: str = ''
    names: List[str] = ()


@dataclass
class TextInfo(TcBaseObj):
    """
    Text Information
    
    :var realName: Real Name
    :var displayName: Display Name
    """
    realName: str = ''
    displayName: str = ''


"""
ExtraInfo
"""
ExtraInfo = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Classification._2015_03.Classification import DependencyAttributeStruct, GetDependencyKeyLOVsResponse
from typing import List
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def getKeyLOVsForDependentAttributes(cls, dependencyAttributeStruct: List[DependencyAttributeStruct]) -> GetDependencyKeyLOVsResponse:
        """
        This operation returns the Classification KeyLOV (stxt) definitions based on the dependency settings on class
        or view attributes.
        
        Use cases:
        If a user changes the value for any attribute in a dependency chain while working in a class containing
        interdependent KeyLOV (stxt) attributes, the other dependent attributes should get configured KeyLOV structures
        and potential auto populated value.
         
        This operation is used to get the value and the KeyLOV definition for the dependent KeyLOV attributes. 
        Consider a class containing two interdependent KeyLOV attributes 
        Country
            United States
            Canada
        State
            Ohio
            California
            Ontario
            Quebec
        - If a user first selects the value for Country attribute say "United States" the State attribute will only
        show states belonging "United States".
        - If a user first selects the value for State attribute say "Ohio"; value for Country attribute will be auto
        populated to "United States". Now, if a user deselects the value for Country attribute; the value for State
        attribute gets deselected as well. 
        
        """
        return cls.execute_soa_method(
            method_name='getKeyLOVsForDependentAttributes',
            library='Classification',
            service_date='2015_03',
            service_name='Classification',
            params={'dependencyAttributeStruct': dependencyAttributeStruct},
            response_cls=GetDependencyKeyLOVsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Administration._2009_10.PersonManagement import GetPersonPropertiesResponse, NameValueMap
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class PersonManagementService(TcService):

    @classmethod
    def getStringProperties(cls, personObjects: List[BusinessObject], attributes: List[str]) -> GetPersonPropertiesResponse:
        """
        This operation retrieves the values of the given properties for the given Person business objects.  The
        supported property names are: locale and time zone. Other property names would be ignored if they are passed in.
        
        Use cases:
        Check the existing time zones or locales of Teamcenter users.
        """
        return cls.execute_soa_method(
            method_name='getStringProperties',
            library='Internal-Administration',
            service_date='2009_10',
            service_name='PersonManagement',
            params={'personObjects': personObjects, 'attributes': attributes},
            response_cls=GetPersonPropertiesResponse,
        )

    @classmethod
    def setStringProperties(cls, personObject: List[BusinessObject], attributes: NameValueMap) -> ServiceData:
        """
        This operation sets the values of the given properties for each Person business object passed in the given
        person object list. The property names specified in attributes map must be locale and timezone. Other
        properties would be ignored. The values of the given properties will be updated on all Person business objects
        given in the list.
        
        Use cases:
        Update locale and timezone properties of a set of Person business objects when those persons are relocated to
        different time zones or different countries
        """
        return cls.execute_soa_method(
            method_name='setStringProperties',
            library='Internal-Administration',
            service_date='2009_10',
            service_name='PersonManagement',
            params={'personObject': personObject, 'attributes': attributes},
            response_cls=ServiceData,
        )



from tcsoa.gen.Translation._2007_06.services import TranslationManagementService as imp0
from tcsoa.utils import TcService


class TranslationManagementService(TcService):
    createTranslationRequest = imp0.createTranslationRequest



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Manufacturing._2012_09.StructureSearch import MapSrchCriteriaToLinesInputInfo, SaveOGLinesInSrchCriteriaResponse, MapSrchCriteriaToLinesResponse, SaveOGLinesInSrchCriteriaInputInfo
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def mapSrchCriteriaToLines(cls, inputs: List[MapSrchCriteriaToLinesInputInfo]) -> MapSrchCriteriaToLinesResponse:
        """
        This operation interprets the given search criteria objects and returns the saved BOM line occurrences from the
        input BOM line scope(s).
        """
        return cls.execute_soa_method(
            method_name='mapSrchCriteriaToLines',
            library='Internal-Manufacturing',
            service_date='2012_09',
            service_name='StructureSearch',
            params={'inputs': inputs},
            response_cls=MapSrchCriteriaToLinesResponse,
        )

    @classmethod
    def saveOGLinesInSrchCriteria(cls, inputs: List[SaveOGLinesInSrchCriteriaInputInfo]) -> SaveOGLinesInSrchCriteriaResponse:
        """
        This operation stores the BOM line occurrences under the Occurrence Group structure in a search criteria object.
        """
        return cls.execute_soa_method(
            method_name='saveOGLinesInSrchCriteria',
            library='Internal-Manufacturing',
            service_date='2012_09',
            service_name='StructureSearch',
            params={'inputs': inputs},
            response_cls=SaveOGLinesInSrchCriteriaResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class DigitalSignSaveInput(TcBaseObj):
    """
    The digitalSigningSave input structure.
    
    :var baseObj: The Dataset object that contains the named reference file to be signed.  This is required.
    :var signedFileTicket: The FMS file ticket for uploading the signed file.  This is required.
    :var createToolName: The tool name that is used to sign the file.  This is required.
    :var validSignature: If this is set to true, then the signature is valid; otherwise, it is not.  If the signing
    tool does not declare the signature is valid, then the base dataset will not be checkin.
    :var userNameWhoSign: The Teamcenter user name that completed the digitally sign operation.
    :var signTime: The timestamp of the digital signature in the signing tool.
    """
    baseObj: Dataset = None
    signedFileTicket: str = ''
    createToolName: str = ''
    validSignature: bool = False
    userNameWhoSign: str = ''
    signTime: str = ''


@dataclass
class DigtalSigningSaveResponse(TcBaseObj):
    """
    Digital signing save operation response
    
    :var basedDataset: If the Dataset is updated successfully, then the Dataset is returned.
    :var serviceData: The partial error list if there is any system errors.
    """
    basedDataset: Dataset = None
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class WhereUsedOutputOccGroup(TcBaseObj):
    """
    This structure contains information for Where Used Output (unpublished for the case of Occurrence Group).
    
    :var inputObject: Input WorkspaceObject object reference for mapping to the output
    :var info: List of 'WhereUsedParentInfoOccGroup' structures
    """
    inputObject: WorkspaceObject = None
    info: List[WhereUsedParentInfoOccGroup] = ()


@dataclass
class WhereUsedParentInfoOccGroup(TcBaseObj):
    """
    This structure contains Where Used Parent Information (unpublished for the case of Occurrence Group).
    
    :var parentOccGroup: parentOccGroup
    :var level: The level at which the parent ItemRevision was found
    """
    parentOccGroup: WorkspaceObject = None
    level: int = 0


@dataclass
class WhereUsedResponseOccGroup(TcBaseObj):
    """
    Where Used Response (unpublished for the case of Occurrence Group)
    
    :var output: List of WhereUsedOutputOccGroup structures
    :var serviceData: Standard ServiceData member
    """
    output: List[WhereUsedOutputOccGroup] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ResourceManagementService(TcService):

    @classmethod
    def createSetupSheets(cls, icoIds: List[str]) -> ServiceData:
        """
        This operation creates setup sheets for Manufacturing Resource Manager resources by using the NX
        Graphicsbuilder functionality.  Starting at the class of each classified resource, a search of the
        Classification hierarchy is conducted in an upward direction until a class is found that the multi-value
        preference 'MRMSetupSheetTemplates' associates with a setup sheet template revision ID.  The NX Graphicsbuilder
        is called to create the setup sheet for the given resource, based on the associated setup sheet template.  The
        resulting setup sheet is associated with the resource's ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='createSetupSheets',
            library='Internal-Manufacturing',
            service_date='2016_03',
            service_name='ResourceManagement',
            params={'icoIds': icoIds},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0PropagationRule
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PropagateDataElement(TcBaseObj):
    """
    Input structure for propagateData service operation.
    Structure contains operation type, source object and the applicable propagation rules.
    
    :var operationType: The operation type. The following are integers legal values against the name of the operation
    - 1     Check-In 
    - 2     Check-Out
    - 3     Create
    - 4     Delete
    - 5     Export
    - 6     Import
    - 7     Revise
    - 8     Save
    - 9     Save As
    - 10     All
    
    
    :var sourceObject: Source object to propagate data from. It can be any business object.
    :var propagationRulesInfo: A List of rules and source objects.
    """
    operationType: int = 0
    sourceObject: BusinessObject = None
    propagationRulesInfo: List[DataToPropagate] = ()


@dataclass
class DataToPropagate(TcBaseObj):
    """
    Structure contains the applicable propagation rules for propagating data and the associated workspace object.
    This structure is one of the elements of propagateDataElement structure, which is the input structure to
    propagateData service operation.
    
    :var applicablePropagationRules: A list of propagation rules objects. Valid value are Fnd0PropagationRule business
    object types. 
    :var associatedObject: Associated source object. It can be any business object.
    """
    applicablePropagationRules: List[Fnd0PropagationRule] = ()
    associatedObject: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.StructureManagement._2014_12.StructureSearch import StructureChangesResponse, ChangeTrackerInput
from tcsoa.gen.StructureManagement._2014_12.Effectivity import CreateOccEffectivityInput, ReleaseStatusEffectivityInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def getStructureChanges(cls, changeTrackerInput: List[ChangeTrackerInput]) -> StructureChangesResponse:
        """
        This operation searches for delta changes in the structure based on different search criteria. The BOMLine
        objects directly or indirectly affected by changes done in Incremental Change context or revision effectivity
        context or occurrence effectivity context that are configured in are returned. The caller must supply the
        BOMLine object(s) that determines the scope for the search in addition to search parameters like intent,
        release status names, and one of unit range or date range. The lines returned are those that are configured by
        the current revision rule. All the BOMLines created/found will be returned as part of ServiceData along with
        incremental changes elements ( in case of IncrementalChange context), or ItemRevisions ( in case of Revision
        effectivity/Occurrence effectivity). All partial errors are grouped by the index of the input vector. This
        operation should be consided when the scope will not yield too many expandlines lines below.
        
        Use cases:
        A user wants to find a set of incremental changes of a particular type and with a specific unit effectivity.
        """
        return cls.execute_soa_method(
            method_name='getStructureChanges',
            library='StructureManagement',
            service_date='2014_12',
            service_name='StructureSearch',
            params={'changeTrackerInput': changeTrackerInput},
            response_cls=StructureChangesResponse,
        )


class EffectivityService(TcService):

    @classmethod
    def createOccurrenceEffectivities(cls, input: List[CreateOccEffectivityInput]) -> ServiceData:
        """
        This operation creates a new effectivity object for each BOMLine  in the list. If the isShared flag is true the
        operation applies same effectivity for all BOMLine in the list.
        
        Use cases:
        - User can create and associate the effectivity with one occurrence by selecting appropriate line in the
        structure and choosing Tools->Effectivity->Occurrence Effectivity->View,Create and Edit.
        - User can create and associate the effectivity with several occurrences by selecting appropriate lines in the
        structure and choosing Tools->Effectivity->Occurrence Effectivity->Create on Multiple BOM Lines, ensure the Use
        shared effectivity check box is cleared so the effectivity is not shared between BOMLines.
        - User can create and associate the same effectivity with several occurrences by selecting appropriate lines in
        the structure and choosing Tools->Effectivity->Occurrence Effectivity->Create on Multiple BOM Lines, ensure the
        Use shared effectivity check box is checked so the effectivity can be shared among all occurrences.
        
        """
        return cls.execute_soa_method(
            method_name='createOccurrenceEffectivities',
            library='StructureManagement',
            service_date='2014_12',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def createReleaseStatusEffectivity(cls, input: List[ReleaseStatusEffectivityInput]) -> ServiceData:
        """
        This operation sets new effectivity on a release status.
        
        Use cases:
        User can configure a release status by choosing Tools->Effectivity->Revision Effectivity, then in the
        Effectivity dialog, select the corresponding release status and click create. Define Unit/Date range to set
        effectivity on the release status.
        """
        return cls.execute_soa_method(
            method_name='createReleaseStatusEffectivity',
            library='StructureManagement',
            service_date='2014_12',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )



from tcsoa.gen.Internal_Configurator._2015_10.services import ConfiguratorManagementService as imp0
from tcsoa.gen.Internal_Configurator._2014_06.services import ConfiguratorManagementService as imp1
from tcsoa.gen.Internal_Configurator._2015_03.services import ConfiguratorManagementService as imp2
from tcsoa.gen.Internal_Configurator._2018_11.services import ConfiguratorManagementService as imp3
from tcsoa.gen.Internal_Configurator._2018_06.services import ConfiguratorManagementService as imp4
from tcsoa.gen.Internal_Configurator._2017_11.services import ConfiguratorManagementService as imp5
from tcsoa.gen.Internal_Configurator._2014_12.services import ConfiguratorManagementService as imp6
from tcsoa.gen.Internal_Configurator._2016_09.services import ConfiguratorManagementService as imp7
from tcsoa.utils import TcService


class ConfiguratorManagementService(TcService):
    convertVariantExpressions = imp0.convertVariantExpressions
    createUpdateVariantRules = imp1.createUpdateVariantRules
    executeSearch = imp1.executeSearch
    fetchNextSearchResults = imp1.fetchNextSearchResults
    getAvailableProductVariability = imp2.getAvailableProductVariability
    getAvailableProductVariability2 = imp0.getAvailableProductVariability
    getConfigurationRosterCoverage = imp3.getConfigurationRosterCoverage
    getConfigurationSessionInfo = imp4.getConfigurationSessionInfo
    getContextBasedVariantExpressions = imp5.getContextBasedVariantExpressions
    getDefaultRules = imp1.getDefaultRules
    getExcludeRules = imp1.getExcludeRules
    getFamilyGroups = imp1.getFamilyGroups
    getIncludeRules = imp1.getIncludeRules
    getModelAndOptionConditions = imp6.getModelAndOptionConditions
    getModelsForProduct = imp1.getModelsForProduct
    getOptionFamilies = imp1.getOptionFamilies
    getOptionValueAvailability = imp7.getOptionValueAvailability
    getOptionValues = imp1.getOptionValues
    getOverlapStates = imp6.getOverlapStates
    getProductDefaults = imp1.getProductDefaults
    getProductDefaults2 = imp0.getProductDefaults
    getRevRulesForConfiguratorContext = imp0.getRevRulesForConfiguratorContext
    getVariability = imp4.getVariability
    getVariantCache = imp1.getVariantCache
    getVariantExpressionDisplayStrings = imp1.getVariantExpressionDisplayStrings
    getVariantExpressions = imp0.getVariantExpressions
    setConfigurationSessionInfo = imp5.setConfigurationSessionInfo
    setVariantExpressions = imp0.setVariantExpressions
    stopSearch = imp1.stopSearch
    updateAdmissibility = imp6.updateAdmissibility
    validateProductConditions = imp4.validateProductConditions
    validateProductConfiguration = imp1.validateProductConfiguration
    validateProductConfiguration2 = imp0.validateProductConfiguration



from __future__ import annotations

from tcsoa.gen.BusinessModeler._2011_06.Constants import GlobalConstantValueResponse2
from typing import List
from tcsoa.utils import TcService


class ConstantsService(TcService):

    @classmethod
    def getGlobalConstantValues2(cls, keys: List[str]) -> GlobalConstantValueResponse2:
        """
        Global constants provide consistent definitions that can be used throughout the system. These constants have
        one or multiple values.  User can retrieve the values of global constants to determine the system behavior
        based on values. This operation gets the values of the named global constants ('keys'). This operation supports
        single value and multi valued global constants. This operation replaces deprecated operation
        'getGlobalConstantValues'.
        """
        return cls.execute_soa_method(
            method_name='getGlobalConstantValues2',
            library='BusinessModeler',
            service_date='2011_06',
            service_name='Constants',
            params={'keys': keys},
            response_cls=GlobalConstantValueResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0SavedBookmark
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.OccurrenceManagement import InfoForAddToBookmarkResp, InfoForAddElemData, InsertLevelInputData, ChildOccurrencesData, InsertLevelResponse, NxtOccsInProdData, NxtChildOccurrencesData, OccsInProdResp, ChildOccurrencesResp, ProductOccurrencesInput, InfoForAddElemResp
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.BOMIndexManagement import CreBOMIndxAdmDataResp, ProcessBomIndexResponse, CreBOMIndxAdmDataInfo, ProcessBomIndexInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class BOMIndexManagementService(TcService):

    @classmethod
    def createBOMIndexAdminData(cls, productContexts: List[CreBOMIndxAdmDataInfo]) -> CreBOMIndxAdmDataResp:
        """
        This operation creates Awb0BOMIndexAdminData instances for the given products and configuration rules and/or
        updates existing Awb0BOMIndexAdminData instances with the new list of variant rules. The state of the
        created/updated Awb0BOMIndexAdminData instances is set to' 'ReadyToIndex''; a scheduled process generates the
        index in the background.The product can be an ItemRevision or CollaborativeDesign(CD). The configuration rules
        comprise of a list of RevisionRule(s) with effectivity information and a list of saved SavedVariantRule(s) with
        their owning ItemRevision(s).
        
        Use cases:
        The system administrator sets up the 'runTcFTSIndexer'('TeamcenterFTSIndexer') to run periodically.
        
        A new product has to be indexed for a given configuration.
        - The client invokes 'createBOMIndexAdminData' with the product and the configuration rules that need to
        applied during indexing. New Awb0BOMIndexAdminData instance gets created for the given product and
        configuration.
        - 'runTcFTSIndexer'  invokes 'processBOMIndex'   to generate BOM index. The input for the 'processBOMIndex'  
        is the Awb0BOMIndexAdminData instances whose states are 'ReadyToIndex', 'IndexGenSuccess',
        'IndexExportSuccess', and 'SolrIndexGenSuccess'.
        
        
        
        A existing product that has already been  indexed have to re- indexed for modified configuration.
        
        - The client invokes 'createBOMIndexAdminData' with the product and the configuration rules that need to
        applied during indexing. The existing Awb0BOMIndexAdminData instance gets updated for the given product and
        changed configuration.
        - 'runTcFTSIndexer'  invokes 'processBOMIndex'   to generate BOM index. The input for the 'processBOMIndex'  
        is the Awb0BOMIndexAdminData instances whose states are 'ReadyToIndex', 'IndexGenSuccess',
        'IndexExportSuccess', and 'SolrIndexGenSuccess'.
        
        """
        return cls.execute_soa_method(
            method_name='createBOMIndexAdminData',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='BOMIndexManagement',
            params={'productContexts': productContexts},
            response_cls=CreBOMIndxAdmDataResp,
        )

    @classmethod
    def processBomIndex(cls, input: ProcessBomIndexInfo) -> ProcessBomIndexResponse:
        """
        This operation manages( generates, updates  or deletes ) indices for the given product and configuration.
        Additionally this operation also generates a 'TCXML' file which represents the configured product indices. The
        product and the configuration rules for which the indices are to be generated or deleted is specified through
        the input Awb0BOMIndexAdminData objects.  
        If the given product was already indexed for the given configuration rules, invoking this operation updates(
        deltas) for the  the indices related to modified structures.  
        
        
        Use cases:
        The system administrator sets up the 'runTcFTSIndexer(TeamcenterFTSIndexer)' to run periodically.
        
        Generation and synchronization of Indices for a configured product
        
        Having set up 'runTcFTSIndexer'  to run periodically, the system administrator performs the following sequence
        when a product has to be indexed for a given set of configuration rules and kept upto date with the changes to
        product structure:
        - Create a text file which contains the information about the product and  RevisionRule and or SavedVariantRule
        that must be used for configuration .
        - Execute 'bomindex_admin' utility using '-function=create' option and provide the text file that contains the
        product and the configuration rules.
        - Executing the 'bomindex_admin' utility persists the product and the configuration rules on
        Awb0BOMIndexAdminData instance with a '"ReadyToIndex"' state.
        
        
        
        The runTcFTSIndexer  utility performs the following sequence to index a configured product:
        
        - Invokes processBomIndex  by passing the 'processingOptions' as 'updateTIEAMTables=true'.
        - Finds all the Awb0BOMIndexAdminData instances in the database using the 'findObjectsByClassAndAttributes'
        operation from the 'Finder' service.
        - Invokes 'processBomIndex'  with Awb0BOMIndexAdminData instances whose states are 'ReadyToIndex',
        'IndexGenSuccess', 'IndexExportSuccess', and 'SolrIndexGenSuccess'.
        - 'runTcFTSIndexer'  utility downloads the 'TCXML' file using the transient file tickets returned in the
        response of 'processBomIndex' call. The 'TCXML' file contains the representation of the configured product
        index that was generated.
        - 'runTcFTSIndexer'  uses the 'TCXML' file to update the 'SOLR' database which will be used during context
        search .
        - On successful update of 'SOLR' database the 'runTcFTSIndexer' utility sets the state as 'SolrIndexGenSuccess'
        on the Awb0BOMIndexAdminData instance using the 'setProperties' operation from the 'Core' service.
        
        
        
        Deletion of Indices for a configured product
        
        The system administrator performs the following sequence when the indices of a configured product are to be
        deleted:
        - Create a text file which contains the information about the product, RevisionRule and or SavedVariantRule for
        which the indices were generated .
        - Execute 'bomindex_admin' utility using '-function=delete' option and provide the text file that contains the
        product and the configuration rules.
        - Executing the 'bomindex_admin' utility finds the Awb0BOMIndexAdminData instance corresponding to the product
        and the configuration rules specified in the input file and sets the status on it as 'MarkedForDeletion'.
        
        
        
        The 'runTcFTSIndexer'  utility performs the following sequence to delete the indices of a configured product:
        - Finds all the Awb0BOMIndexAdminData instances in the database using the 'findObjectsByClassAndAttributes'
        operation from the 'Finder' service.
        - Invokes 'processBomIndex'  with Awb0BOMIndexAdminData instances whose states are 'MarkedForDeletion'.
        - 'runTcFTSIndexer'  utility checks the state of the Awb0BOMIndexAdminData instances returned by the
        'processBomIndex' . For all those instances with state 'IndexDelSuccess' the utility deletes the indices in
        'SOLR'.
        - 'runTcFTSIndexer'  now deletes the Awb0BOMIndexAdminData instances whose status is  'SolrIndexDelSuccess' 
        using 'deleteObjects' operation from 'Core' service.
        
        """
        return cls.execute_soa_method(
            method_name='processBomIndex',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='BOMIndexManagement',
            params={'input': input},
            response_cls=ProcessBomIndexResponse,
        )


class OccurrenceManagementService(TcService):

    @classmethod
    def getChildOccurrences(cls, input: List[ChildOccurrencesData]) -> ChildOccurrencesResp:
        """
        This operation retrieves the child occurrences for the set of input parent occurrences. The child occurrences
        may be a the same level or different levels based on input 'firstLevelOnly'.The number of occurrences retuned
        depends on the paging configuration given in input.
        """
        return cls.execute_soa_method(
            method_name='getChildOccurrences',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ChildOccurrencesResp,
        )

    @classmethod
    def getInfoForAddElement(cls, getInfoForElementIn: InfoForAddElemData) -> InfoForAddElemResp:
        """
        This operation retreives information required for creating an Awb0Element under product specified in
        Awb0ProductContextInfo or Awb0Element. The operation also returns allowable type name(s) to search existing
        object through Full Text Search. The preferred type is the parent type if that is allowed.
        """
        return cls.execute_soa_method(
            method_name='getInfoForAddElement',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'getInfoForElementIn': getInfoForElementIn},
            response_cls=InfoForAddElemResp,
        )

    @classmethod
    def getInfoForAddToBookmark(cls) -> InfoForAddToBookmarkResp:
        """
        This operation retreives information required for adding a product to a Awb0SavedBookmark. The operation also
        returns allowable type name(s) to search an existing object through Full Text Search.
        """
        return cls.execute_soa_method(
            method_name='getInfoForAddToBookmark',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={},
            response_cls=InfoForAddToBookmarkResp,
        )

    @classmethod
    def getNextChildOccurrences(cls, input: List[NxtChildOccurrencesData]) -> ChildOccurrencesResp:
        """
        This operation gets the next set of child occurrences for given parent occurrence(s).
        """
        return cls.execute_soa_method(
            method_name='getNextChildOccurrences',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ChildOccurrencesResp,
        )

    @classmethod
    def getNextOccurrencesInProduct(cls, inputData: List[NxtOccsInProdData]) -> OccsInProdResp:
        """
        Get the next set of occurrences in the product. This method should called by passing in the cursor object
        returned by a call to 'getOccurrenceInProduct' or a previous call to 'getNextOccurrencesInProduct'.
        """
        return cls.execute_soa_method(
            method_name='getNextOccurrencesInProduct',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccsInProdResp,
        )

    @classmethod
    def getOccurrencesInProduct(cls, inputData: List[ProductOccurrencesInput]) -> OccsInProdResp:
        """
        This operation fetches occurrences in a product. The inputs allow providing the configuration and filter
        information that will help idenfity the occurrences to return. The objects attached to the occurrences can be
        returned by specifying the relation types and object types to return. This operation only returns an page of
        occurrences. Call the method 'GetNextOccurrencesInProduct' to load the next set of occurrences in the product.
        """
        return cls.execute_soa_method(
            method_name='getOccurrencesInProduct',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccsInProdResp,
        )

    @classmethod
    def insertLevel(cls, input: List[InsertLevelInputData]) -> InsertLevelResponse:
        """
        This operation creates an Awb0Element object for the given Item object and inserts it as a parent of the given
        Awb0Element objects.
        """
        return cls.execute_soa_method(
            method_name='insertLevel',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=InsertLevelResponse,
        )

    @classmethod
    def updateSavedBookmark(cls, savedBookmarkObjects: List[Awb0SavedBookmark]) -> ServiceData:
        """
        This operation updates the Awb0SavedBookmark objects based on corresponding Awb0Autobookmark object.
        """
        return cls.execute_soa_method(
            method_name='updateSavedBookmark',
            library='Internal-ActiveWorkspaceBom',
            service_date='2012_10',
            service_name='OccurrenceManagement',
            params={'savedBookmarkObjects': savedBookmarkObjects},
            response_cls=ServiceData,
        )
